"""
DJI D-Gamut Colourspace
=======================

Defines the *DJI D-Gamut* colourspace:

-   :attr:`colour.models.RGB_COLOURSPACE_DJI_D_GAMUT`.

References
----------
-   :cite:`DJI2017` : Dji. (2017). White Paper on D-Log and D-Gamut of DJI
    Cinema Color System (pp. 1-5).
    https://dl.djicdn.com/downloads/zenmuse+x7/20171010/\
D-Log_D-Gamut_Whitepaper.pdf
"""

from __future__ import annotations

import numpy as np

from colour.colorimetry import CCS_ILLUMINANTS
from colour.hints import NDArray
from colour.models.rgb import (
    RGB_Colourspace,
    log_encoding_DJIDLog,
    log_decoding_DJIDLog,
)

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "PRIMARIES_DJI_D_GAMUT",
    "WHITEPOINT_NAME_DJI_D_GAMUT",
    "CCS_WHITEPOINT_DJI_D_GAMUT",
    "MATRIX_DJI_D_GAMUT_TO_XYZ",
    "MATRIX_XYZ_TO_DJI_D_GAMUT",
    "RGB_COLOURSPACE_DJI_D_GAMUT",
]

PRIMARIES_DJI_D_GAMUT: NDArray = np.array(
    [
        [0.71, 0.31],
        [0.21, 0.88],
        [0.09, -0.08],
    ]
)
"""*DJI D-Gamut* colourspace primaries."""

WHITEPOINT_NAME_DJI_D_GAMUT: str = "D65"
"""*DJI D-Gamut* colourspace whitepoint name."""

CCS_WHITEPOINT_DJI_D_GAMUT: NDArray = CCS_ILLUMINANTS[
    "CIE 1931 2 Degree Standard Observer"
][WHITEPOINT_NAME_DJI_D_GAMUT]
"""*DJI D-Gamut* colourspace whitepoint chromaticity coordinates."""

MATRIX_DJI_D_GAMUT_TO_XYZ: NDArray = np.array(
    [
        [0.6482, 0.1940, 0.1082],
        [0.2830, 0.8132, -0.0962],
        [-0.0183, -0.0832, 1.1903],
    ]
)
"""*DJI D-Gamut* colourspace to *CIE XYZ* tristimulus values matrix."""

MATRIX_XYZ_TO_DJI_D_GAMUT: NDArray = np.array(
    [
        [1.7257, -0.4314, -0.1917],
        [-0.6025, 1.3906, 0.1671],
        [-0.0156, 0.0905, 0.8489],
    ]
)
"""*CIE XYZ* tristimulus values to *DJI D-Gamut* colourspace matrix."""

RGB_COLOURSPACE_DJI_D_GAMUT: RGB_Colourspace = RGB_Colourspace(
    "DJI D-Gamut",
    PRIMARIES_DJI_D_GAMUT,
    CCS_WHITEPOINT_DJI_D_GAMUT,
    WHITEPOINT_NAME_DJI_D_GAMUT,
    MATRIX_DJI_D_GAMUT_TO_XYZ,
    MATRIX_XYZ_TO_DJI_D_GAMUT,
    log_encoding_DJIDLog,
    log_decoding_DJIDLog,
)
RGB_COLOURSPACE_DJI_D_GAMUT.__doc__ = """
*DJI_D-Gamut* colourspace.

References
----------
:cite:`DJI2017`
"""



"""
Fujifilm F-Log Log Encoding
===========================

Defines the *Fujifilm F-Log* log encoding:

-   :func:`colour.models.log_encoding_FLog`
-   :func:`colour.models.log_decoding_FLog`

References
----------
-   :cite:`Fujifilm2016` : Fujifilm. (2016). F-Log Data Sheet Ver.1.0 (pp.
    1-4). https://www.fujifilm.com/support/digital_cameras/software/lut/pdf/\
F-Log_DataSheet_E_Ver.1.0.pdf
"""

from __future__ import annotations

import numpy as np

from colour.hints import (
    Boolean,
    FloatingOrArrayLike,
    FloatingOrNDArray,
    Integer,
)
from colour.models.rgb.transfer_functions import full_to_legal, legal_to_full
from colour.utilities import Structure, as_float, from_range_1, to_domain_1

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - http://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "CONSTANTS_FLOG",
    "log_encoding_FLog",
    "log_decoding_FLog",
]

CONSTANTS_FLOG: Structure = Structure(
    cut1=0.00089,
    cut2=0.100537775223865,
    a=0.555556,
    b=0.009468,
    c=0.344676,
    d=0.790453,
    e=8.735631,
    f=0.092864,
)
"""*Fujifilm F-Log* colourspace constants."""


def log_encoding_FLog(
    in_r: FloatingOrArrayLike,
    bit_depth: Integer = 10,
    out_normalised_code_value: Boolean = True,
    in_reflection: Boolean = True,
    constants: Structure = CONSTANTS_FLOG,
) -> FloatingOrNDArray:
    """
    Define the *Fujifilm F-Log* log encoding curve / opto-electronic transfer
    function.

    Parameters
    ----------
    in_r
        Linear reflection data :math`in`.
    bit_depth
        Bit depth used for conversion.
    out_normalised_code_value
        Whether the non-linear *Fujifilm F-Log* data :math:`out` is encoded as
        normalised code values.
    in_reflection
        Whether the light level :math`in` to a camera is reflection.
    constants
        *Fujifilm F-Log* constants.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.ndarray`
        Non-linear data :math:`out`.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``in_r``   | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``out_r``  | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    References
    ----------
    :cite:`Fujifilm2016`

    Examples
    --------
    >>> log_encoding_FLog(0.18)  # doctest: +ELLIPSIS
    0.4593184...

    The values of *2-2. F-Log Code Value* table in :cite:`Fujifilm2016` are
    obtained as follows:

    >>> x = np.array([0, 18, 90]) / 100
    >>> np.around(log_encoding_FLog(x, 10, False) * 100, 1)
    array([  3.5,  46.3,  73.2])
    >>> np.around(log_encoding_FLog(x) * (2 ** 10 - 1)).astype(np.int)
    array([ 95, 470, 705])
    """

    in_r = to_domain_1(in_r)

    if not in_reflection:
        in_r = in_r * 0.9

    cut1 = constants.cut1
    a = constants.a
    b = constants.b
    c = constants.c
    d = constants.d
    e = constants.e
    f = constants.f

    out_r = np.where(
        in_r < cut1,
        e * in_r + f,
        c * np.log10(a * in_r + b) + d,
    )

    out_r_cv = (
        out_r if out_normalised_code_value else legal_to_full(out_r, bit_depth)
    )

    return as_float(from_range_1(out_r_cv))


def log_decoding_FLog(
    out_r: FloatingOrArrayLike,
    bit_depth: Integer = 10,
    in_normalised_code_value: Boolean = True,
    out_reflection: Boolean = True,
    constants: Structure = CONSTANTS_FLOG,
) -> FloatingOrNDArray:
    """
    Define the *Fujifilm F-Log* log decoding curve / electro-optical transfer
    function.

    Parameters
    ----------
    out_r
        Non-linear data :math:`out`.
    bit_depth
        Bit depth used for conversion.
    in_normalised_code_value
        Whether the non-linear *Fujifilm F-Log* data :math:`out` is encoded as
        normalised code values.
    out_reflection
        Whether the light level :math`in` to a camera is reflection.
    constants
        *Fujifilm F-Log* constants.

    Returns
    -------
    :class:`numpy.floating` or :class:`numpy.ndarray`
        Linear reflection data :math`in`.

    Notes
    -----
    +------------+-----------------------+---------------+
    | **Domain** | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``out_r``  | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    +------------+-----------------------+---------------+
    | **Range**  | **Scale - Reference** | **Scale - 1** |
    +============+=======================+===============+
    | ``in_r``   | [0, 1]                | [0, 1]        |
    +------------+-----------------------+---------------+

    References
    ----------
    :cite:`Fujifilm2016`

    Examples
    --------
    >>> log_decoding_FLog(0.45931845866162124)  # doctest: +ELLIPSIS
    0.1800000...
    """

    out_r = to_domain_1(out_r)

    out_r = (
        out_r if in_normalised_code_value else full_to_legal(out_r, bit_depth)
    )

    cut2 = constants.cut2
    a = constants.a
    b = constants.b
    c = constants.c
    d = constants.d
    e = constants.e
    f = constants.f

    in_r = np.where(
        out_r < cut2,
        (out_r - f) / e,
        (10 ** ((out_r - d) / c)) / a - b / a,
    )

    if not out_reflection:
        in_r = in_r / 0.9

    return as_float(from_range_1(in_r))



"""
Defines the unit tests for the
:mod:`colour.models.rgb.transfer_functions.itur_bt_2100` module.
"""

import numpy as np
import unittest

from colour.models.rgb.transfer_functions import (
    oetf_PQ_BT2100,
    oetf_inverse_PQ_BT2100,
    eotf_PQ_BT2100,
    eotf_inverse_PQ_BT2100,
    ootf_PQ_BT2100,
    ootf_inverse_PQ_BT2100,
    oetf_HLG_BT2100,
    oetf_inverse_HLG_BT2100,
)
from colour.models.rgb.transfer_functions.itur_bt_2100 import (
    eotf_HLG_BT2100_1,
    eotf_HLG_BT2100_2,
    eotf_inverse_HLG_BT2100_1,
    eotf_inverse_HLG_BT2100_2,
    ootf_HLG_BT2100_1,
    ootf_HLG_BT2100_2,
    ootf_inverse_HLG_BT2100_1,
    ootf_inverse_HLG_BT2100_2,
)
from colour.models.rgb.transfer_functions.itur_bt_2100 import (
    gamma_function_HLG_BT2100,
)
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestOetf_PQ_BT2100",
    "TestOetf_inverse_PQ_BT2100",
    "TestEotf_PQ_BT2100",
    "TestEotf_inverse_PQ_BT2100",
    "TestOotf_PQ_BT2100",
    "TestOotf_inverse_PQ_BT2100",
    "TestGamma_function_HLG_BT2100",
    "TestOetf_HLG_BT2100",
    "TestOetf_inverse_HLG_BT2100",
    "TestEotf_HLG_BT2100_1",
    "TestEotf_HLG_BT2100_2",
    "TestEotf_inverse_HLG_BT2100_1",
    "TestEotf_inverse_HLG_BT2100_2",
    "TestOotf_HLG_BT2100_1",
    "TestOotf_HLG_BT2100_2",
    "TestOotf_inverse_HLG_BT2100_1",
    "TestOotf_inverse_HLG_BT2100_2",
]


class TestOetf_PQ_BT2100(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_PQ_BT2100` definition unit tests methods.
    """

    def test_oetf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_PQ_BT2100` definition.
        """

        self.assertAlmostEqual(
            oetf_PQ_BT2100(0.0), 0.000000730955903, places=7
        )

        self.assertAlmostEqual(
            oetf_PQ_BT2100(0.1), 0.724769816665726, places=7
        )

        self.assertAlmostEqual(
            oetf_PQ_BT2100(1.0), 0.999999934308041, places=7
        )

    def test_n_dimensional_oetf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_PQ_BT2100` definition n-dimensional arrays support.
        """

        E = 0.1
        E_p = oetf_PQ_BT2100(E)

        E = np.tile(E, 6)
        E_p = np.tile(E_p, 6)
        np.testing.assert_almost_equal(oetf_PQ_BT2100(E), E_p, decimal=7)

        E = np.reshape(E, (2, 3))
        E_p = np.reshape(E_p, (2, 3))
        np.testing.assert_almost_equal(oetf_PQ_BT2100(E), E_p, decimal=7)

        E = np.reshape(E, (2, 3, 1))
        E_p = np.reshape(E_p, (2, 3, 1))
        np.testing.assert_almost_equal(oetf_PQ_BT2100(E), E_p, decimal=7)

    def test_domain_range_scale_oetf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_PQ_BT2100` definition domain and range scale support.
        """

        E = 0.1
        E_p = oetf_PQ_BT2100(E)

        d_r = (("reference", 1), ("1", 1), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    oetf_PQ_BT2100(E * factor), E_p * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_oetf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_PQ_BT2100` definition nan support.
        """

        oetf_PQ_BT2100(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestOetf_inverse_PQ_BT2100(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_inverse_PQ_BT2100` definition unit tests methods.
    """

    def test_oetf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_inverse_PQ_BT2100` definition.
        """

        self.assertAlmostEqual(
            oetf_inverse_PQ_BT2100(0.000000730955903), 0.0, places=7
        )

        self.assertAlmostEqual(
            oetf_inverse_PQ_BT2100(0.724769816665726), 0.1, places=7
        )

        self.assertAlmostEqual(
            oetf_inverse_PQ_BT2100(0.999999934308041), 1.0, places=7
        )

    def test_n_dimensional_oetf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_inverse_PQ_BT2100` definition n-dimensional arrays support.
        """

        E_p = 0.724769816665726
        E = oetf_inverse_PQ_BT2100(E_p)

        E_p = np.tile(E_p, 6)
        E = np.tile(E, 6)
        np.testing.assert_almost_equal(
            oetf_inverse_PQ_BT2100(E_p), E, decimal=7
        )

        E_p = np.reshape(E_p, (2, 3))
        E = np.reshape(E, (2, 3))
        np.testing.assert_almost_equal(
            oetf_inverse_PQ_BT2100(E_p), E, decimal=7
        )

        E_p = np.reshape(E_p, (2, 3, 1))
        E = np.reshape(E, (2, 3, 1))
        np.testing.assert_almost_equal(
            oetf_inverse_PQ_BT2100(E_p), E, decimal=7
        )

    def test_domain_range_scale_oetf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_inverse_PQ_BT2100` definition domain and range scale support.
        """

        E_p = 0.724769816665726
        E = oetf_inverse_PQ_BT2100(E_p)

        d_r = (("reference", 1), ("1", 1), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    oetf_inverse_PQ_BT2100(E_p * factor), E * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_oetf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_inverse_PQ_BT2100` definition nan support.
        """

        oetf_inverse_PQ_BT2100(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan])
        )


class TestEotf_PQ_BT2100(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_PQ_BT2100` definition unit tests methods.
    """

    def test_eotf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_PQ_BT2100` definition.
        """

        self.assertAlmostEqual(eotf_PQ_BT2100(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            eotf_PQ_BT2100(0.724769816665726), 779.98836083408537, places=7
        )

        self.assertAlmostEqual(eotf_PQ_BT2100(1.0), 10000.0, places=7)

    def test_n_dimensional_eotf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_PQ_BT2100` definition n-dimensional arrays support.
        """

        E_p = 0.724769816665726
        F_D = eotf_PQ_BT2100(E_p)

        E_p = np.tile(E_p, 6)
        F_D = np.tile(F_D, 6)
        np.testing.assert_almost_equal(eotf_PQ_BT2100(E_p), F_D, decimal=7)

        E_p = np.reshape(E_p, (2, 3))
        F_D = np.reshape(F_D, (2, 3))
        np.testing.assert_almost_equal(eotf_PQ_BT2100(E_p), F_D, decimal=7)

        E_p = np.reshape(E_p, (2, 3, 1))
        F_D = np.reshape(F_D, (2, 3, 1))
        np.testing.assert_almost_equal(eotf_PQ_BT2100(E_p), F_D, decimal=7)

    def test_domain_range_scale_eotf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_PQ_BT2100` definition domain and range scale support.
        """

        E_p = 0.724769816665726
        F_D = eotf_PQ_BT2100(E_p)

        d_r = (("reference", 1), ("1", 1), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    eotf_PQ_BT2100(E_p * factor), F_D * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_eotf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_PQ_BT2100` definition nan support.
        """

        eotf_PQ_BT2100(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestEotf_inverse_PQ_BT2100(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_PQ_BT2100` definition unit tests methods.
    """

    def test_eotf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_PQ_BT2100` definition.
        """

        self.assertAlmostEqual(
            eotf_inverse_PQ_BT2100(0.0), 0.000000730955903, places=7
        )

        self.assertAlmostEqual(
            eotf_inverse_PQ_BT2100(779.98836083408537),
            0.724769816665726,
            places=7,
        )

        self.assertAlmostEqual(eotf_inverse_PQ_BT2100(10000.0), 1.0, places=7)

    def test_n_dimensional_eotf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_PQ_BT2100` definition n-dimensional arrays support.
        """

        F_D = 779.98836083408537
        E_p = eotf_inverse_PQ_BT2100(F_D)

        F_D = np.tile(F_D, 6)
        E_p = np.tile(E_p, 6)
        np.testing.assert_almost_equal(
            eotf_inverse_PQ_BT2100(F_D), E_p, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3))
        E_p = np.reshape(E_p, (2, 3))
        np.testing.assert_almost_equal(
            eotf_inverse_PQ_BT2100(F_D), E_p, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3, 1))
        E_p = np.reshape(E_p, (2, 3, 1))
        np.testing.assert_almost_equal(
            eotf_inverse_PQ_BT2100(F_D), E_p, decimal=7
        )

    def test_domain_range_scale_eotf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_PQ_BT2100` definition domain and range scale support.
        """

        F_D = 779.98836083408537
        E_p = eotf_inverse_PQ_BT2100(F_D)

        d_r = (("reference", 1), ("1", 1), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    eotf_inverse_PQ_BT2100(F_D * factor),
                    E_p * factor,
                    decimal=7,
                )

    @ignore_numpy_errors
    def test_nan_eotf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_PQ_BT2100` definition nan support.
        """

        eotf_inverse_PQ_BT2100(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan])
        )


class TestOotf_PQ_BT2100(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_PQ_BT2100` definition unit tests methods.
    """

    def test_ootf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_PQ_BT2100` definition.
        """

        self.assertAlmostEqual(ootf_PQ_BT2100(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            ootf_PQ_BT2100(0.1), 779.98836083411584, places=7
        )

        self.assertAlmostEqual(
            ootf_PQ_BT2100(1.0), 9999.993723673924300, places=7
        )

    def test_n_dimensional_ootf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_PQ_BT2100` definition n-dimensional arrays support.
        """

        E = 0.1
        F_D = ootf_PQ_BT2100(E)

        E = np.tile(E, 6)
        F_D = np.tile(F_D, 6)
        np.testing.assert_almost_equal(ootf_PQ_BT2100(E), F_D, decimal=7)

        E = np.reshape(E, (2, 3))
        F_D = np.reshape(F_D, (2, 3))
        np.testing.assert_almost_equal(ootf_PQ_BT2100(E), F_D, decimal=7)

        E = np.reshape(E, (2, 3, 1))
        F_D = np.reshape(F_D, (2, 3, 1))
        np.testing.assert_almost_equal(ootf_PQ_BT2100(E), F_D, decimal=7)

    def test_domain_range_scale_ootf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_PQ_BT2100` definition domain and range scale support.
        """

        E = 0.1
        F_D = ootf_PQ_BT2100(E)

        d_r = (("reference", 1), ("1", 1), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    ootf_PQ_BT2100(E * factor), F_D * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_ootf_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_PQ_BT2100` definition nan support.
        """

        ootf_PQ_BT2100(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestOotf_inverse_PQ_BT2100(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_PQ_BT2100` definition unit tests methods.
    """

    def test_ootf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_PQ_BT2100` definition.
        """

        self.assertAlmostEqual(ootf_inverse_PQ_BT2100(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            ootf_inverse_PQ_BT2100(779.98836083411584), 0.1, places=7
        )

        self.assertAlmostEqual(
            ootf_inverse_PQ_BT2100(9999.993723673924300), 1.0, places=7
        )

    def test_n_dimensional_ootf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_PQ_BT2100` definition n-dimensional arrays support.
        """

        F_D = 779.98836083411584
        E = ootf_inverse_PQ_BT2100(F_D)

        F_D = np.tile(F_D, 6)
        E = np.tile(E, 6)
        np.testing.assert_almost_equal(
            ootf_inverse_PQ_BT2100(F_D), E, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3))
        E = np.reshape(E, (2, 3))
        np.testing.assert_almost_equal(
            ootf_inverse_PQ_BT2100(F_D), E, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3, 1))
        E = np.reshape(E, (2, 3, 1))
        np.testing.assert_almost_equal(
            ootf_inverse_PQ_BT2100(F_D), E, decimal=7
        )

    def test_domain_range_scale_ootf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_PQ_BT2100` definition domain and range scale support.
        """

        F_D = 779.98836083411584
        E = ootf_inverse_PQ_BT2100(F_D)

        d_r = (("reference", 1), ("1", 1), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    ootf_inverse_PQ_BT2100(F_D * factor), E * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_ootf_inverse_PQ_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_PQ_BT2100` definition nan support.
        """

        ootf_inverse_PQ_BT2100(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan])
        )


class TestGamma_function_HLG_BT2100(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
gamma_function_HLG_BT2100` definition unit tests methods.
    """

    def test_gamma_function_HLG_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
gamma_function_HLG_BT2100` definition.
        """

        self.assertAlmostEqual(
            gamma_function_HLG_BT2100(1000.0), 1.2, places=7
        )

        self.assertAlmostEqual(
            gamma_function_HLG_BT2100(2000.0), 1.326432598178872, places=7
        )

        self.assertAlmostEqual(
            gamma_function_HLG_BT2100(4000.0), 1.452865196357744, places=7
        )

        self.assertAlmostEqual(
            gamma_function_HLG_BT2100(10000.0), 1.619999999999999, places=7
        )


class TestOetf_HLG_BT2100(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_HLG_BT2100` definition unit tests methods.
    """

    def test_oetf_HLG_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_HLG_BT2100` definition.
        """

        self.assertAlmostEqual(oetf_HLG_BT2100(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            oetf_HLG_BT2100(0.18 / 12), 0.212132034355964, places=7
        )

        self.assertAlmostEqual(
            oetf_HLG_BT2100(1.0), 0.999999995536569, places=7
        )

    def test_n_dimensional_oetf_HLG_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_HLG_BT2100` definition n-dimensional arrays support.
        """

        E = 0.18 / 12
        E_p = oetf_HLG_BT2100(E)

        E = np.tile(E, 6)
        E_p = np.tile(E_p, 6)
        np.testing.assert_almost_equal(oetf_HLG_BT2100(E), E_p, decimal=7)

        E = np.reshape(E, (2, 3))
        E_p = np.reshape(E_p, (2, 3))
        np.testing.assert_almost_equal(oetf_HLG_BT2100(E), E_p, decimal=7)

        E = np.reshape(E, (2, 3, 1))
        E_p = np.reshape(E_p, (2, 3, 1))
        np.testing.assert_almost_equal(oetf_HLG_BT2100(E), E_p, decimal=7)

    def test_domain_range_scale_oetf_HLG_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_HLG_BT2100` definition domain and range scale support.
        """

        E = 0.18 / 12
        E_p = oetf_HLG_BT2100(E)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    oetf_HLG_BT2100(E * factor), E_p * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_oetf_HLG_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_HLG_BT2100` definition nan support.
        """

        oetf_HLG_BT2100(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestOetf_inverse_HLG_BT2100(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_inverse_HLG_BT2100` definition unit tests methods.
    """

    def test_oetf_inverse_HLG_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_inverse_HLG_BT2100` definition.
        """

        self.assertAlmostEqual(oetf_inverse_HLG_BT2100(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            oetf_inverse_HLG_BT2100(0.212132034355964), 0.18 / 12, places=7
        )

        self.assertAlmostEqual(
            oetf_inverse_HLG_BT2100(0.999999995536569), 1.0, places=7
        )

    def test_n_dimensional_oetf_inverse_HLG_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_inverse_HLG_BT2100` definition n-dimensional arrays support.
        """

        E_p = 0.212132034355964
        E = oetf_inverse_HLG_BT2100(E_p)

        E_p = np.tile(E_p, 6)
        E = np.tile(E, 6)
        np.testing.assert_almost_equal(
            oetf_inverse_HLG_BT2100(E_p), E, decimal=7
        )

        E_p = np.reshape(E_p, (2, 3))
        E = np.reshape(E, (2, 3))
        np.testing.assert_almost_equal(
            oetf_inverse_HLG_BT2100(E_p), E, decimal=7
        )

        E_p = np.reshape(E_p, (2, 3, 1))
        E = np.reshape(E, (2, 3, 1))
        np.testing.assert_almost_equal(
            oetf_inverse_HLG_BT2100(E_p), E, decimal=7
        )

    def test_domain_range_scale_oetf_inverse_HLG_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_inverse_HLG_BT2100` definition domain and range scale support.
        """

        E_p = 0.212132034355964
        E = oetf_inverse_HLG_BT2100(E_p)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    oetf_inverse_HLG_BT2100(E_p * factor),
                    E * factor,
                    decimal=7,
                )

    @ignore_numpy_errors
    def test_nan_oetf_inverse_HLG_BT2100(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
oetf_inverse_HLG_BT2100` definition nan support.
        """

        oetf_inverse_HLG_BT2100(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan])
        )


class TestEotf_HLG_BT2100_1(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_HLG_BT2100_1` definition unit tests methods.
    """

    def test_eotf_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_HLG_BT2100_1` definition.
        """

        self.assertAlmostEqual(eotf_HLG_BT2100_1(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            eotf_HLG_BT2100_1(0.212132034355964), 6.476039825649814, places=7
        )

        self.assertAlmostEqual(
            eotf_HLG_BT2100_1(1.0), 1000.000032321769100, places=7
        )

        self.assertAlmostEqual(
            eotf_HLG_BT2100_1(0.212132034355964, 0.001, 10000, 1.4),
            27.96039175299561,
            places=7,
        )

    def test_n_dimensional_eotf_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_HLG_BT2100_1` definition n-dimensional arrays support.
        """

        E_p = 0.212132034355964
        F_D = eotf_HLG_BT2100_1(E_p)

        E_p = np.tile(E_p, 6)
        F_D = np.tile(F_D, 6)
        np.testing.assert_almost_equal(eotf_HLG_BT2100_1(E_p), F_D, decimal=7)

        E_p = np.reshape(E_p, (2, 3))
        F_D = np.reshape(F_D, (2, 3))
        np.testing.assert_almost_equal(eotf_HLG_BT2100_1(E_p), F_D, decimal=7)

        E_p = np.reshape(E_p, (2, 3, 1))
        F_D = np.reshape(F_D, (2, 3, 1))
        np.testing.assert_almost_equal(eotf_HLG_BT2100_1(E_p), F_D, decimal=7)

        E_p = np.reshape(E_p, (6, 1))
        F_D = np.reshape(F_D, (6, 1))
        np.testing.assert_almost_equal(eotf_HLG_BT2100_1(E_p), F_D, decimal=7)

        E_p = np.array([0.25, 0.50, 0.75])
        F_D = np.array([12.49759413, 49.99037650, 158.94693786])
        np.testing.assert_almost_equal(eotf_HLG_BT2100_1(E_p), F_D, decimal=7)

        E_p = np.tile(E_p, (6, 1))
        F_D = np.tile(F_D, (6, 1))
        np.testing.assert_almost_equal(eotf_HLG_BT2100_1(E_p), F_D, decimal=7)

        E_p = np.reshape(E_p, (2, 3, 3))
        F_D = np.reshape(F_D, (2, 3, 3))
        np.testing.assert_almost_equal(eotf_HLG_BT2100_1(E_p), F_D, decimal=7)

    def test_domain_range_scale_eotf_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_HLG_BT2100_1` definition domain and range scale support.
        """

        E_p = 0.212132034355964
        F_D = eotf_HLG_BT2100_1(E_p)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    eotf_HLG_BT2100_1(E_p * factor), F_D * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_eotf_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_HLG_BT2100_1` definition nan support.
        """

        eotf_HLG_BT2100_1(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestEotf_HLG_BT2100_2(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_HLG_BT2100_2` definition unit tests methods.
    """

    def test_eotf_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_HLG_BT2100_2` definition.
        """

        self.assertAlmostEqual(eotf_HLG_BT2100_2(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            eotf_HLG_BT2100_2(0.212132034355964), 6.476039825649814, places=7
        )

        self.assertAlmostEqual(
            eotf_HLG_BT2100_2(1.0), 1000.000032321769100, places=7
        )

        self.assertAlmostEqual(
            eotf_HLG_BT2100_2(0.212132034355964, 0.001, 10000, 1.4),
            29.581261576946076,
            places=7,
        )

    def test_n_dimensional_eotf_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_HLG_BT2100_2` definition n-dimensional arrays support.
        """

        E_p = 0.212132034355964
        F_D = eotf_HLG_BT2100_2(E_p)

        E_p = np.tile(E_p, 6)
        F_D = np.tile(F_D, 6)
        np.testing.assert_almost_equal(eotf_HLG_BT2100_2(E_p), F_D, decimal=7)

        E_p = np.reshape(E_p, (2, 3))
        F_D = np.reshape(F_D, (2, 3))
        np.testing.assert_almost_equal(eotf_HLG_BT2100_2(E_p), F_D, decimal=7)

        E_p = np.reshape(E_p, (2, 3, 1))
        F_D = np.reshape(F_D, (2, 3, 1))
        np.testing.assert_almost_equal(eotf_HLG_BT2100_2(E_p), F_D, decimal=7)

        E_p = np.reshape(E_p, (6, 1))
        F_D = np.reshape(F_D, (6, 1))
        np.testing.assert_almost_equal(eotf_HLG_BT2100_2(E_p), F_D, decimal=7)

        E_p = np.array([0.25, 0.50, 0.75])
        F_D = np.array([12.49759413, 49.99037650, 158.94693786])
        np.testing.assert_almost_equal(eotf_HLG_BT2100_2(E_p), F_D, decimal=7)

        E_p = np.tile(E_p, (6, 1))
        F_D = np.tile(F_D, (6, 1))
        np.testing.assert_almost_equal(eotf_HLG_BT2100_2(E_p), F_D, decimal=7)

        E_p = np.reshape(E_p, (2, 3, 3))
        F_D = np.reshape(F_D, (2, 3, 3))
        np.testing.assert_almost_equal(eotf_HLG_BT2100_2(E_p), F_D, decimal=7)

    def test_domain_range_scale_eotf_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_HLG_BT2100_2` definition domain and range scale support.
        """

        E_p = 0.212132034355964
        F_D = eotf_HLG_BT2100_2(E_p)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    eotf_HLG_BT2100_2(E_p * factor), F_D * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_eotf_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_HLG_BT2100_2` definition nan support.
        """

        eotf_HLG_BT2100_2(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestEotf_inverse_HLG_BT2100_1(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_HLG_BT2100_1` definition unit tests methods.
    """

    def test_eotf_inverse_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_HLG_BT2100_1` definition.
        """

        self.assertAlmostEqual(eotf_inverse_HLG_BT2100_1(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            eotf_inverse_HLG_BT2100_1(6.476039825649814),
            0.212132034355964,
            places=7,
        )

        self.assertAlmostEqual(
            eotf_inverse_HLG_BT2100_1(1000.000032321769100), 1.0, places=7
        )

        self.assertAlmostEqual(
            eotf_inverse_HLG_BT2100_1(27.96039175299561, 0.001, 10000, 1.4),
            0.212132034355964,
            places=7,
        )

    def test_n_dimensional_eotf_inverse_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_HLG_BT2100_1` definition n-dimensional arrays support.
        """

        F_D = 6.476039825649814
        E_p = eotf_inverse_HLG_BT2100_1(F_D)

        F_D = np.tile(F_D, 6)
        E_p = np.tile(E_p, 6)
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_1(F_D), E_p, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3))
        E_p = np.reshape(E_p, (2, 3))
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_1(F_D), E_p, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3, 1))
        E_p = np.reshape(E_p, (2, 3, 1))
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_1(F_D), E_p, decimal=7
        )

        F_D = np.reshape(F_D, (6, 1))
        E_p = np.reshape(E_p, (6, 1))
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_1(F_D), E_p, decimal=7
        )

        F_D = np.array([12.49759413, 49.99037650, 158.94693786])
        E_p = np.array([0.25, 0.50, 0.75])
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_1(F_D), E_p, decimal=7
        )

        F_D = np.tile(F_D, (6, 1))
        E_p = np.tile(E_p, (6, 1))
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_1(F_D), E_p, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3, 3))
        E_p = np.reshape(E_p, (2, 3, 3))
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_1(F_D), E_p, decimal=7
        )

    def test_domain_range_scale_eotf_inverse_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_HLG_BT2100_1` definition domain and range scale support.
        """

        F_D = 6.476039825649814
        E_p = eotf_inverse_HLG_BT2100_1(F_D)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    eotf_inverse_HLG_BT2100_1(F_D * factor),
                    E_p * factor,
                    decimal=7,
                )

    @ignore_numpy_errors
    def test_nan_eotf_inverse_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_HLG_BT2100_1` definition nan support.
        """

        eotf_inverse_HLG_BT2100_1(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan])
        )


class TestEotf_inverse_HLG_BT2100_2(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_HLG_BT2100_2` definition unit tests methods.
    """

    def test_eotf_inverse_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_HLG_BT2100_2` definition.
        """

        self.assertAlmostEqual(eotf_inverse_HLG_BT2100_2(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            eotf_inverse_HLG_BT2100_2(6.476039825649814),
            0.212132034355964,
            places=7,
        )

        self.assertAlmostEqual(
            eotf_inverse_HLG_BT2100_2(1000.000032321769100), 1.0, places=7
        )

        self.assertAlmostEqual(
            eotf_inverse_HLG_BT2100_2(29.581261576946076, 0.001, 10000, 1.4),
            0.212132034355964,
            places=7,
        )

    def test_n_dimensional_eotf_inverse_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_HLG_BT2100_2` definition n-dimensional arrays support.
        """

        F_D = 6.476039825649814
        E_p = eotf_inverse_HLG_BT2100_2(F_D)

        F_D = np.tile(F_D, 6)
        E_p = np.tile(E_p, 6)
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_2(F_D), E_p, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3))
        E_p = np.reshape(E_p, (2, 3))
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_2(F_D), E_p, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3, 1))
        E_p = np.reshape(E_p, (2, 3, 1))
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_2(F_D), E_p, decimal=7
        )

        F_D = np.reshape(F_D, (6, 1))
        E_p = np.reshape(E_p, (6, 1))
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_2(F_D), E_p, decimal=7
        )

        F_D = np.array([12.49759413, 49.99037650, 158.94693786])
        E_p = np.array([0.25, 0.50, 0.75])
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_2(F_D), E_p, decimal=7
        )

        F_D = np.tile(F_D, (6, 1))
        E_p = np.tile(E_p, (6, 1))
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_2(F_D), E_p, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3, 3))
        E_p = np.reshape(E_p, (2, 3, 3))
        np.testing.assert_almost_equal(
            eotf_inverse_HLG_BT2100_2(F_D), E_p, decimal=7
        )

    def test_domain_range_scale_eotf_inverse_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_HLG_BT2100_2` definition domain and range scale support.
        """

        F_D = 6.476039825649814
        E_p = eotf_inverse_HLG_BT2100_2(F_D)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    eotf_inverse_HLG_BT2100_2(F_D * factor),
                    E_p * factor,
                    decimal=7,
                )

    @ignore_numpy_errors
    def test_nan_eotf_inverse_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
eotf_inverse_HLG_BT2100_2` definition nan support.
        """

        eotf_inverse_HLG_BT2100_2(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan])
        )


class TestOotf_HLG_BT2100_1(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_HLG_BT2100_1` definition unit tests methods.
    """

    def test_ootf_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_HLG_BT2100_1` definition.
        """

        self.assertAlmostEqual(ootf_HLG_BT2100_1(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            ootf_HLG_BT2100_1(0.1), 63.095734448019336, places=7
        )

        self.assertAlmostEqual(ootf_HLG_BT2100_1(1.0), 1000.0, places=7)

        self.assertAlmostEqual(
            ootf_HLG_BT2100_1(0.1, 0.001, 10000, 1.4),
            398.108130742780300,
            places=7,
        )

        a = np.array(
            [
                [45.884942278760597, 0.000000000000000, -45.884942278760597],
                [
                    -63.095734448019336,
                    -63.095734448019336,
                    -63.095734448019336,
                ],
                [63.095734448019336, 63.095734448019336, 63.095734448019336],
                [51.320396090100672, -51.320396090100672, 51.320396090100672],
            ],
        )
        np.testing.assert_almost_equal(
            ootf_HLG_BT2100_1(
                np.array(
                    [
                        [0.1, 0.0, -0.1],
                        [-0.1, -0.1, -0.1],
                        [0.1, 0.1, 0.1],
                        [0.1, -0.1, 0.1],
                    ]
                )
            ),
            a,
            decimal=7,
        )

    def test_n_dimensional_ootf_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_HLG_BT2100_1` definition n-dimensional arrays support.
        """

        E = 0.1
        F_D = ootf_HLG_BT2100_1(E)

        E = np.tile(E, 6)
        F_D = np.tile(F_D, 6)
        np.testing.assert_almost_equal(ootf_HLG_BT2100_1(E), F_D, decimal=7)

        E = np.reshape(E, (2, 3))
        F_D = np.reshape(F_D, (2, 3))
        np.testing.assert_almost_equal(ootf_HLG_BT2100_1(E), F_D, decimal=7)

        E = np.reshape(E, (2, 3, 1))
        F_D = np.reshape(F_D, (2, 3, 1))
        np.testing.assert_almost_equal(ootf_HLG_BT2100_1(E), F_D, decimal=7)

        E = np.reshape(E, (6, 1))
        F_D = np.reshape(F_D, (6, 1))
        np.testing.assert_almost_equal(ootf_HLG_BT2100_1(E), F_D, decimal=7)

        E = np.array([0.25, 0.50, 0.75])
        F_D = np.array([213.01897444, 426.03794887, 639.05692331])
        np.testing.assert_almost_equal(ootf_HLG_BT2100_1(E), F_D, decimal=7)

        E = np.tile(E, (6, 1))
        F_D = np.tile(F_D, (6, 1))
        np.testing.assert_almost_equal(ootf_HLG_BT2100_1(E), F_D, decimal=7)

        E = np.reshape(E, (2, 3, 3))
        F_D = np.reshape(F_D, (2, 3, 3))
        np.testing.assert_almost_equal(ootf_HLG_BT2100_1(E), F_D, decimal=7)

    def test_domain_range_scale_ootf_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_HLG_BT2100_1` definition domain and range scale support.
        """

        E = 0.1
        F_D = ootf_HLG_BT2100_1(E)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    ootf_HLG_BT2100_1(E * factor), F_D * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_ootf_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_HLG_BT2100_1` definition nan support.
        """

        ootf_HLG_BT2100_1(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestOotf_HLG_BT2100_2(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_HLG_BT2100_2` definition unit tests methods.
    """

    def test_ootf_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_HLG_BT2100_2` definition.
        """

        self.assertAlmostEqual(ootf_HLG_BT2100_2(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            ootf_HLG_BT2100_2(0.1), 63.095734448019336, places=7
        )

        self.assertAlmostEqual(ootf_HLG_BT2100_2(1.0), 1000.0, places=7)

        self.assertAlmostEqual(
            ootf_HLG_BT2100_2(0.1, 10000, 1.4), 398.107170553497380, places=7
        )

        a = np.array(
            [
                [45.884942278760597, 0.000000000000000, -45.884942278760597],
                [
                    -63.095734448019336,
                    -63.095734448019336,
                    -63.095734448019336,
                ],
                [63.095734448019336, 63.095734448019336, 63.095734448019336],
                [51.320396090100672, -51.320396090100672, 51.320396090100672],
            ],
        )
        np.testing.assert_almost_equal(
            ootf_HLG_BT2100_2(
                np.array(
                    [
                        [0.1, 0.0, -0.1],
                        [-0.1, -0.1, -0.1],
                        [0.1, 0.1, 0.1],
                        [0.1, -0.1, 0.1],
                    ]
                )
            ),
            a,
            decimal=7,
        )

    def test_n_dimensional_ootf_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_HLG_BT2100_2` definition n-dimensional arrays support.
        """

        E = 0.1
        F_D = ootf_HLG_BT2100_2(E)

        E = np.tile(E, 6)
        F_D = np.tile(F_D, 6)
        np.testing.assert_almost_equal(ootf_HLG_BT2100_2(E), F_D, decimal=7)

        E = np.reshape(E, (2, 3))
        F_D = np.reshape(F_D, (2, 3))
        np.testing.assert_almost_equal(ootf_HLG_BT2100_2(E), F_D, decimal=7)

        E = np.reshape(E, (2, 3, 1))
        F_D = np.reshape(F_D, (2, 3, 1))
        np.testing.assert_almost_equal(ootf_HLG_BT2100_2(E), F_D, decimal=7)

        E = np.reshape(E, (6, 1))
        F_D = np.reshape(F_D, (6, 1))
        np.testing.assert_almost_equal(ootf_HLG_BT2100_2(E), F_D, decimal=7)

        E = np.array([0.25, 0.50, 0.75])
        F_D = np.array([213.01897444, 426.03794887, 639.05692331])
        np.testing.assert_almost_equal(ootf_HLG_BT2100_2(E), F_D, decimal=7)

        E = np.tile(E, (6, 1))
        F_D = np.tile(F_D, (6, 1))
        np.testing.assert_almost_equal(ootf_HLG_BT2100_2(E), F_D, decimal=7)

        E = np.reshape(E, (2, 3, 3))
        F_D = np.reshape(F_D, (2, 3, 3))
        np.testing.assert_almost_equal(ootf_HLG_BT2100_2(E), F_D, decimal=7)

    def test_domain_range_scale_ootf_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_HLG_BT2100_2` definition domain and range scale support.
        """

        E = 0.1
        F_D = ootf_HLG_BT2100_1(E)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    ootf_HLG_BT2100_1(E * factor), F_D * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_ootf_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_HLG_BT2100_1` definition nan support.
        """

        ootf_HLG_BT2100_1(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestOotf_inverse_HLG_BT2100_1(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_HLG_BT2100_1` definition unit tests methods.
    """

    def test_ootf_inverse_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_HLG_BT2100_1` definition.
        """

        self.assertAlmostEqual(ootf_inverse_HLG_BT2100_1(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            ootf_inverse_HLG_BT2100_1(63.095734448019336), 0.1, places=7
        )

        self.assertAlmostEqual(
            ootf_inverse_HLG_BT2100_1(1000.0), 1.0, places=7
        )

        self.assertAlmostEqual(
            ootf_inverse_HLG_BT2100_1(398.108130742780300, 0.001, 10000, 1.4),
            0.1,
            places=7,
        )

        a = np.array(
            [
                [45.884942278760597, 0.000000000000000, -45.884942278760597],
                [
                    -63.095734448019336,
                    -63.095734448019336,
                    -63.095734448019336,
                ],
                [63.095734448019336, 63.095734448019336, 63.095734448019336],
                [51.320396090100672, -51.320396090100672, 51.320396090100672],
            ]
        )
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_1(a),
            np.array(
                [
                    [0.1, 0.0, -0.1],
                    [-0.1, -0.1, -0.1],
                    [0.1, 0.1, 0.1],
                    [0.1, -0.1, 0.1],
                ]
            ),
            decimal=7,
        )

    def test_n_dimensional_ootf_inverse_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_HLG_BT2100_1` definition n-dimensional arrays support.
        """

        F_D = 63.095734448019336
        E = ootf_inverse_HLG_BT2100_1(F_D)

        F_D = np.tile(F_D, 6)
        E = np.tile(E, 6)
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_1(F_D), E, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3))
        E = np.reshape(E, (2, 3))
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_1(F_D), E, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3, 1))
        E = np.reshape(E, (2, 3, 1))
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_1(F_D), E, decimal=7
        )

        F_D = np.reshape(F_D, (6, 1))
        E = np.reshape(E, (6, 1))
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_1(F_D), E, decimal=7
        )

        F_D = np.array([213.01897444, 426.03794887, 639.05692331])
        E = np.array([0.25, 0.50, 0.75])
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_1(F_D), E, decimal=7
        )

        F_D = np.tile(F_D, (6, 1))
        E = np.tile(E, (6, 1))
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_1(F_D), E, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3, 3))
        E = np.reshape(E, (2, 3, 3))
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_1(F_D), E, decimal=7
        )

    def test_domain_range_scale_ootf_inverse_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_HLG_BT2100_1` definition domain and range scale support.
        """

        F_D = 63.095734448019336
        E = ootf_inverse_HLG_BT2100_1(F_D)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    ootf_inverse_HLG_BT2100_1(F_D * factor),
                    E * factor,
                    decimal=7,
                )

    @ignore_numpy_errors
    def test_nan_ootf_inverse_HLG_BT2100_1(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_HLG_BT2100_1` definition nan support.
        """

        ootf_inverse_HLG_BT2100_1(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan])
        )


class TestOotf_inverse_HLG_BT2100_2(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_HLG_BT2100_2` definition unit tests methods.
    """

    def test_ootf_inverse_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_HLG_BT2100_2` definition.
        """

        self.assertAlmostEqual(ootf_inverse_HLG_BT2100_2(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            ootf_inverse_HLG_BT2100_2(63.095734448019336), 0.1, places=7
        )

        self.assertAlmostEqual(
            ootf_inverse_HLG_BT2100_2(1000.0), 1.0, places=7
        )

        self.assertAlmostEqual(
            ootf_inverse_HLG_BT2100_2(398.107170553497380, 10000, 1.4),
            0.1,
            places=7,
        )

        a = np.array(
            [
                [45.884942278760597, 0.000000000000000, -45.884942278760597],
                [
                    -63.095734448019336,
                    -63.095734448019336,
                    -63.095734448019336,
                ],
                [63.095734448019336, 63.095734448019336, 63.095734448019336],
                [51.320396090100672, -51.320396090100672, 51.320396090100672],
            ]
        )
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_2(a),
            np.array(
                [
                    [0.1, 0.0, -0.1],
                    [-0.1, -0.1, -0.1],
                    [0.1, 0.1, 0.1],
                    [0.1, -0.1, 0.1],
                ]
            ),
            decimal=7,
        )

    def test_n_dimensional_ootf_inverse_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_HLG_BT2100_2` definition n-dimensional arrays support.
        """

        F_D = 63.095734448019336
        E = ootf_inverse_HLG_BT2100_2(F_D)

        F_D = np.tile(F_D, 6)
        E = np.tile(E, 6)
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_2(F_D), E, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3))
        E = np.reshape(E, (2, 3))
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_2(F_D), E, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3, 1))
        E = np.reshape(E, (2, 3, 1))
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_2(F_D), E, decimal=7
        )

        F_D = np.reshape(F_D, (6, 1))
        E = np.reshape(E, (6, 1))
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_2(F_D), E, decimal=7
        )

        F_D = np.array([213.01897444, 426.03794887, 639.05692331])
        E = np.array([0.25, 0.50, 0.75])
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_2(F_D), E, decimal=7
        )

        F_D = np.tile(F_D, (6, 1))
        E = np.tile(E, (6, 1))
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_2(F_D), E, decimal=7
        )

        F_D = np.reshape(F_D, (2, 3, 3))
        E = np.reshape(E, (2, 3, 3))
        np.testing.assert_almost_equal(
            ootf_inverse_HLG_BT2100_2(F_D), E, decimal=7
        )

    def test_domain_range_scale_ootf_inverse_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_HLG_BT2100_2` definition domain and range scale support.
        """

        F_D = 63.095734448019336
        E = ootf_inverse_HLG_BT2100_2(F_D)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    ootf_inverse_HLG_BT2100_2(F_D * factor),
                    E * factor,
                    decimal=7,
                )

    @ignore_numpy_errors
    def test_nan_ootf_inverse_HLG_BT2100_2(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_2100.\
ootf_inverse_HLG_BT2100_2` definition nan support.
        """

        ootf_inverse_HLG_BT2100_2(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan])
        )


if __name__ == "__main__":
    unittest.main()



"""Defines the unit tests for the :mod:`colour.temperature.mccamy1992` module."""

import numpy as np
import unittest
from itertools import permutations

from colour.temperature import xy_to_CCT_McCamy1992, CCT_to_xy_McCamy1992
from colour.utilities import ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "Testxy_to_CCT_McCamy1992",
    "TestCCT_to_xy_McCamy1992",
]


class Testxy_to_CCT_McCamy1992(unittest.TestCase):
    """
    Define :func:`colour.temperature.mccamy1992.xy_to_CCT_McCamy1992`
    definition unit tests methods.
    """

    def test_xy_to_CCT_McCamy1992(self):
        """
        Test :func:`colour.temperature.mccamy1992.xy_to_CCT_McCamy1992`
        definition.
        """

        self.assertAlmostEqual(
            xy_to_CCT_McCamy1992(np.array([0.31270, 0.32900])),
            6505.08059131,
            places=7,
        )

        self.assertAlmostEqual(
            xy_to_CCT_McCamy1992(np.array([0.44757, 0.40745])),
            2857.28961266,
            places=7,
        )

        self.assertAlmostEqual(
            xy_to_CCT_McCamy1992(
                np.array([0.252520939374083, 0.252220883926284])
            ),
            19501.61953130,
            places=7,
        )

    def test_n_dimensional_xy_to_CCT_McCamy1992(self):
        """
        Test :func:`colour.temperature.mccamy1992.xy_to_CCT_McCamy1992`
        definition n-dimensional arrays support.
        """

        xy = np.array([0.31270, 0.32900])
        CCT = xy_to_CCT_McCamy1992(xy)

        xy = np.tile(xy, (6, 1))
        CCT = np.tile(CCT, 6)
        np.testing.assert_almost_equal(
            xy_to_CCT_McCamy1992(xy), CCT, decimal=7
        )

        xy = np.reshape(xy, (2, 3, 2))
        CCT = np.reshape(CCT, (2, 3))
        np.testing.assert_almost_equal(
            xy_to_CCT_McCamy1992(xy), CCT, decimal=7
        )

    @ignore_numpy_errors
    def test_nan_xy_to_CCT_McCamy1992(self):
        """
        Test :func:`colour.temperature.mccamy1992.xy_to_CCT_McCamy1992`
        definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=2))
        for case in cases:
            xy_to_CCT_McCamy1992(case)


class TestCCT_to_xy_McCamy1992(unittest.TestCase):
    """
    Define :func:`colour.temperature.mccamy1992.CCT_to_xy_McCamy1992`
    definition unit tests methods.
    """

    def test_CCT_to_xy_McCamy1992(self):
        """
        Test :func:`colour.temperature.mccamy1992.CCT_to_xy_McCamy1992`
        definition.
        """

        np.testing.assert_almost_equal(
            CCT_to_xy_McCamy1992(6505.08059131, {"method": "Nelder-Mead"}),
            np.array([0.31269945, 0.32900411]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            CCT_to_xy_McCamy1992(2857.28961266, {"method": "Nelder-Mead"}),
            np.array([0.42350314, 0.36129253]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            CCT_to_xy_McCamy1992(19501.61953130, {"method": "Nelder-Mead"}),
            np.array([0.11173782, 0.36987375]),
            decimal=7,
        )

    def test_n_dimensional_CCT_to_xy_McCamy1992(self):
        """
        Test :func:`colour.temperature.mccamy1992.CCT_to_xy_McCamy1992`
        definition n-dimensional arrays support.
        """

        CCT = 6505.08059131
        xy = CCT_to_xy_McCamy1992(CCT)

        CCT = np.tile(CCT, 6)
        xy = np.tile(xy, (6, 1))
        np.testing.assert_almost_equal(
            CCT_to_xy_McCamy1992(CCT), xy, decimal=7
        )

        CCT = np.reshape(CCT, (2, 3))
        xy = np.reshape(xy, (2, 3, 2))
        np.testing.assert_almost_equal(
            CCT_to_xy_McCamy1992(CCT), xy, decimal=7
        )

    @ignore_numpy_errors
    def test_nan_CCT_to_xy_McCamy1992(self):
        """
        Test :func:`colour.temperature.mccamy1992.CCT_to_xy_McCamy1992`
        definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=2))
        for case in cases:
            CCT_to_xy_McCamy1992(case)


if __name__ == "__main__":
    unittest.main()



"""
Spectral Distributions of Lenses
================================

Defines the spectral distributions of lenses.

Each lens data is in the form of :class:`dict` class instance of
:class:`colour.SpectralDistribution` classes as follows::

    {'name': SpectralDistribution, ..., 'name': SpectralDistribution}

The following *lenses* are available:

-   ISO Standard Lens

References
----------
-   :cite:`InternationalOrganizationforStandardization2002` : International
    Organization for Standardization. (2002). INTERNATIONAL STANDARD ISO
    7589-2002 - Photography - Illuminants for sensitometry - Specifications for
    daylight, incandescent tungsten and printer.
"""

from __future__ import annotations

from functools import partial

from colour.colorimetry import SpectralDistribution
from colour.hints import Dict
from colour.utilities import LazyCaseInsensitiveMapping

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "DATA_LENSES_ISO",
    "SDS_LENSES_ISO",
    "SDS_LENSES",
]

DATA_LENSES_ISO: Dict = {
    "ISO Standard Lens": {
        350: 0.00,
        360: 0.07,
        370: 0.23,
        380: 0.42,
        390: 0.60,
        400: 0.74,
        410: 0.83,
        420: 0.88,
        430: 0.91,
        440: 0.94,
        450: 0.95,
        460: 0.97,
        470: 0.98,
        480: 0.98,
        490: 0.99,
        500: 0.99,
        510: 1.00,
        520: 1.00,
        530: 1.00,
        540: 1.00,
        550: 1.00,
        560: 1.00,
        570: 1.00,
        580: 1.00,
        590: 0.99,
        600: 0.99,
        610: 0.99,
        620: 0.98,
        630: 0.98,
        640: 0.97,
        650: 0.97,
        660: 0.96,
        670: 0.95,
        680: 0.94,
        690: 0.94,
    }
}

SDS_LENSES_ISO: LazyCaseInsensitiveMapping = LazyCaseInsensitiveMapping(
    {
        "ISO Standard Lens": partial(
            SpectralDistribution,
            DATA_LENSES_ISO["ISO Standard Lens"],
            name="ISO Standard Lens",
        ),
    }
)
SDS_LENSES_ISO.__doc__ = """
Spectral distributions of *ISO* lenses.

References
----------
:cite:`InternationalOrganizationforStandardization2002`
"""

SDS_LENSES: LazyCaseInsensitiveMapping = LazyCaseInsensitiveMapping(
    SDS_LENSES_ISO
)
SDS_LENSES.__doc__ = """
Spectral distributions of lenses.

References
----------
:cite:`InternationalOrganizationforStandardization2002`
"""



"""
Chromaticity Coordinates of the Colour Checkers
===============================================

Defines the chromaticity coordinates of the colour checkers.

Each colour checker data is in the form of an :class:`dict` class instance of
24 samples as follows::

    {'name': 'xyY', ..., 'name': 'xyY'}

The following colour checkers are available:

-   :attr:`colour.characterisation.datasets.colour_checkers.\
chromaticity_coordinates.CCS_COLORCHECKER1976`: *ColorChecker Classic*
    developed by *McCamy et al. (1976)* at Macbeth, a Division of Kollmorgen.
-   :attr:`colour.characterisation.datasets.colour_checkers.\
chromaticity_coordinates.CCS_COLORCHECKER2005`: *ColorChecker Classic*
    reference data from *GretagMacbeth* published in 2005.
-   :attr:`colour.characterisation.datasets.colour_checkers.\
chromaticity_coordinates.CCS_BABELCOLOR_AVERAGE`: Average data derived from
    measurements of 30 *ColorChecker Classic* charts.
-   :attr:`colour.characterisation.datasets.colour_checkers.\
chromaticity_coordinates.CCS_COLORCHECKER24_BEFORE_NOV2014`:
    *ColorChecker Classic* reference data from *X-Rite* published in 2015 and
    matching the data from *GretagMacbeth* published in 2005.
-   :attr:`colour.characterisation.datasets.colour_checkers.\
chromaticity_coordinates.CCS_COLORCHECKER24_AFTER_NOV2014`:
    *ColorChecker Classic* reference data from *X-Rite* published in 2015 and
    matching the *ColorChecker Classic* edition after November 2014.

References
----------
-   :cite:`BabelColor2012b` : BabelColor. (2012). The ColorChecker (since
    1976!). Retrieved September 26, 2014, from
    http://www.babelcolor.com/main_level/ColorChecker.htm
-   :cite:`BabelColor2012c` : BabelColor. (2012). ColorChecker RGB and
    spectra.
    http://www.babelcolor.com/download/ColorChecker_RGB_and_spectra.xls
-   :cite:`X-Rite2016` : X-Rite. (2016). New color specifications for
    ColorChecker SG and Classic Charts. Retrieved October 29, 2018, from
    http://xritephoto.com/ph_product_overview.aspx?ID=938&Action=Support&\
SupportID=5884#
"""

from __future__ import annotations

import numpy as np
from collections import namedtuple

from colour.colorimetry import CCS_ILLUMINANTS
from colour.hints import Dict, NDArray, Tuple
from colour.models import Lab_to_XYZ, XYZ_to_xyY
from colour.utilities import CaseInsensitiveMapping

__author__ = "Colour Developers, Danny Pascale "
__copyright__ = "Copyright 2013 Colour Developers"
__copyright__ += ", "
__copyright__ += (
    "BabelColor ColorChecker data: Copyright (C) 2004-2012 Danny Pascale "
    "(www.babelcolor.com); used by permission."
)
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "ColourChecker",
    "SAMPLE_LABELS_COLORCHECKER_CLASSIC",
    "DATA_COLORCHECKER1976",
    "CCS_ILLUMINANT_COLORCHECKER1976",
    "CCS_COLORCHECKER1976",
    "DATA_COLORCHECKER2005",
    "CCS_ILLUMINANT_COLORCHECKER2005",
    "CCS_COLORCHECKER2005",
    "DATA_BABELCOLOR_AVERAGE",
    "CCS_ILLUMINANT_BABELCOLOR_AVERAGE",
    "CCS_BABELCOLOR_AVERAGE",
    "DATA_COLORCHECKER24_BEFORE_NOV2014_CIE_LAB",
    "DATA_COLORCHECKER24_BEFORE_NOV2014",
    "CCS_ILLUMINANT_COLORCHECKER24_BEFORE_NOV2014",
    "CCS_COLORCHECKER24_BEFORE_NOV2014",
    "DATA_COLORCHECKER24_AFTER_NOV2014",
    "DATA_COLORCHECKER24_AFTER_NOV2014",
    "CCS_ILLUMINANT_COLORCHECKER24_AFTER_NOV2014",
    "CCS_COLORCHECKER24_AFTER_NOV2014",
    "CCS_COLOURCHECKERS",
]


class ColourChecker(
    namedtuple("ColourChecker", ("name", "data", "illuminant"))
):
    """
    *Colour Checker* data.

    Parameters
    ----------
    name
        *Colour Checker* name.
    data
        Chromaticity coordinates in *CIE xyY* colourspace.
    illuminant
        *Colour Checker* illuminant chromaticity coordinates.
    """


SAMPLE_LABELS_COLORCHECKER_CLASSIC: Tuple = (
    "dark skin",
    "light skin",
    "blue sky",
    "foliage",
    "blue flower",
    "bluish green",
    "orange",
    "purplish blue",
    "moderate red",
    "purple",
    "yellow green",
    "orange yellow",
    "blue",
    "green",
    "red",
    "yellow",
    "magenta",
    "cyan",
    "white 9.5 (.05 D)",
    "neutral 8 (.23 D)",
    "neutral 6.5 (.44 D)",
    "neutral 5 (.70 D)",
    "neutral 3.5 (1.05 D)",
    "black 2 (1.5 D)",
)
"""*ColorChecker Classic* illuminant."""

DATA_COLORCHECKER1976: Dict = dict(
    zip(
        SAMPLE_LABELS_COLORCHECKER_CLASSIC,
        [
            np.array([0.4002, 0.3504, 0.1005]),
            np.array([0.3773, 0.3446, 0.3582]),
            np.array([0.2470, 0.2514, 0.1933]),
            np.array([0.3372, 0.4220, 0.1329]),
            np.array([0.2651, 0.2400, 0.2427]),
            np.array([0.2608, 0.3430, 0.4306]),
            np.array([0.5060, 0.4070, 0.3005]),
            np.array([0.2110, 0.1750, 0.1200]),
            np.array([0.4533, 0.3058, 0.1977]),
            np.array([0.2845, 0.2020, 0.0656]),
            np.array([0.3800, 0.4887, 0.4429]),
            np.array([0.4729, 0.4375, 0.4306]),
            np.array([0.1866, 0.1285, 0.0611]),
            np.array([0.3046, 0.4782, 0.2339]),
            np.array([0.5385, 0.3129, 0.1200]),
            np.array([0.4480, 0.4703, 0.5910]),
            np.array([0.3635, 0.2325, 0.1977]),
            np.array([0.1958, 0.2519, 0.1977]),
            np.array([0.3101, 0.3163, 0.9001]),
            np.array([0.3101, 0.3163, 0.5910]),
            np.array([0.3101, 0.3163, 0.3620]),
            np.array([0.3101, 0.3163, 0.1977]),
            np.array([0.3101, 0.3163, 0.0900]),
            np.array([0.3101, 0.3163, 0.0313]),
        ],
    )
)

CCS_ILLUMINANT_COLORCHECKER1976: NDArray = CCS_ILLUMINANTS[
    "CIE 1931 2 Degree Standard Observer"
]["C"]
"""*ColorChecker Classic 1976* illuminant."""

CCS_COLORCHECKER1976: ColourChecker = ColourChecker(
    "ColorChecker 1976", DATA_COLORCHECKER1976, CCS_ILLUMINANT_COLORCHECKER1976
)
"""
*ColorChecker Classic* developed by *McCamy et al.* (1976) at Macbeth, a
Division of Kollmorgen.
"""

DATA_COLORCHECKER2005: Dict = dict(
    zip(
        SAMPLE_LABELS_COLORCHECKER_CLASSIC,
        [
            np.array([0.4316, 0.3777, 0.1008]),
            np.array([0.4197, 0.3744, 0.3495]),
            np.array([0.2760, 0.3016, 0.1836]),
            np.array([0.3703, 0.4499, 0.1325]),
            np.array([0.2999, 0.2856, 0.2304]),
            np.array([0.2848, 0.3911, 0.4178]),
            np.array([0.5295, 0.4055, 0.3118]),
            np.array([0.2305, 0.2106, 0.1126]),
            np.array([0.5012, 0.3273, 0.1938]),
            np.array([0.3319, 0.2482, 0.0637]),
            np.array([0.3984, 0.5008, 0.4446]),
            np.array([0.4957, 0.4427, 0.4357]),
            np.array([0.2018, 0.1692, 0.0575]),
            np.array([0.3253, 0.5032, 0.2318]),
            np.array([0.5686, 0.3303, 0.1257]),
            np.array([0.4697, 0.4734, 0.5981]),
            np.array([0.4159, 0.2688, 0.2009]),
            np.array([0.2131, 0.3023, 0.1930]),
            np.array([0.3469, 0.3608, 0.9131]),
            np.array([0.3440, 0.3584, 0.5894]),
            np.array([0.3432, 0.3581, 0.3632]),
            np.array([0.3446, 0.3579, 0.1915]),
            np.array([0.3401, 0.3548, 0.0883]),
            np.array([0.3406, 0.3537, 0.0311]),
        ],
    )
)

CCS_ILLUMINANT_COLORCHECKER2005: NDArray = CCS_ILLUMINANTS[
    "CIE 1931 2 Degree Standard Observer"
]["ICC D50"]
"""*ColorChecker Classic 2005* illuminant."""

CCS_COLORCHECKER2005: ColourChecker = ColourChecker(
    "ColorChecker 2005", DATA_COLORCHECKER2005, CCS_ILLUMINANT_COLORCHECKER2005
)
"""*ColorChecker Classic* data from *GretagMacbeth (2005)*."""
DATA_BABELCOLOR_AVERAGE: Dict = dict(
    zip(
        SAMPLE_LABELS_COLORCHECKER_CLASSIC,
        [
            np.array([0.4325, 0.3788, 0.1034]),
            np.array([0.4191, 0.3748, 0.3525]),
            np.array([0.2761, 0.3004, 0.1847]),
            np.array([0.3700, 0.4501, 0.1335]),
            np.array([0.3020, 0.2877, 0.2324]),
            np.array([0.2856, 0.3910, 0.4174]),
            np.array([0.5291, 0.4075, 0.3117]),
            np.array([0.2339, 0.2155, 0.1140]),
            np.array([0.5008, 0.3293, 0.1979]),
            np.array([0.3326, 0.2556, 0.0644]),
            np.array([0.3989, 0.4998, 0.4435]),
            np.array([0.4962, 0.4428, 0.4358]),
            np.array([0.2040, 0.1696, 0.0579]),
            np.array([0.3270, 0.5033, 0.2307]),
            np.array([0.5709, 0.3298, 0.1268]),
            np.array([0.4694, 0.4732, 0.6081]),
            np.array([0.4177, 0.2704, 0.2007]),
            np.array([0.2151, 0.3037, 0.1903]),
            np.array([0.3488, 0.3628, 0.9129]),
            np.array([0.3451, 0.3596, 0.5885]),
            np.array([0.3446, 0.3590, 0.3595]),
            np.array([0.3438, 0.3589, 0.1912]),
            np.array([0.3423, 0.3576, 0.0893]),
            np.array([0.3439, 0.3565, 0.0320]),
        ],
    )
)

CCS_ILLUMINANT_BABELCOLOR_AVERAGE: NDArray = CCS_ILLUMINANTS[
    "CIE 1931 2 Degree Standard Observer"
]["ICC D50"]
"""*BabelColor Average* illuminant."""

CCS_BABELCOLOR_AVERAGE: ColourChecker = ColourChecker(
    "BabelColor Average",
    DATA_BABELCOLOR_AVERAGE,
    CCS_ILLUMINANT_BABELCOLOR_AVERAGE,
)
"""Average data derived from measurements of 30 *ColorChecker Classic* charts."""

DATA_COLORCHECKER24_BEFORE_NOV2014_CIE_LAB: Dict = dict(
    zip(
        SAMPLE_LABELS_COLORCHECKER_CLASSIC,
        [
            np.array([37.986, 13.555, 14.059]),
            np.array([65.711, 18.13, 17.81]),
            np.array([49.927, -4.88, -21.905]),
            np.array([43.139, -13.095, 21.905]),
            np.array([55.112, 8.844, -25.399]),
            np.array([70.719, -33.397, -0.199]),
            np.array([62.661, 36.067, 57.096]),
            np.array([40.02, 10.41, -45.964]),
            np.array([51.124, 48.239, 16.248]),
            np.array([30.325, 22.976, -21.587]),
            np.array([72.532, -23.709, 57.255]),
            np.array([71.941, 19.363, 67.857]),
            np.array([28.778, 14.179, -50.297]),
            np.array([55.261, -38.342, 31.37]),
            np.array([42.101, 53.378, 28.19]),
            np.array([81.733, 4.039, 79.819]),
            np.array([51.935, 49.986, -14.574]),
            np.array([51.038, -28.631, -28.638]),
            np.array([96.539, -0.425, 1.186]),
            np.array([81.257, -0.638, -0.335]),
            np.array([66.766, -0.734, -0.504]),
            np.array([50.867, -0.153, -0.27]),
            np.array([35.656, -0.421, -1.231]),
            np.array([20.461, -0.079, -0.973]),
        ],
    )
)

DATA_COLORCHECKER24_BEFORE_NOV2014: Dict = dict(
    zip(
        SAMPLE_LABELS_COLORCHECKER_CLASSIC,
        XYZ_to_xyY(
            Lab_to_XYZ(
                list(DATA_COLORCHECKER24_BEFORE_NOV2014_CIE_LAB.values()),
                CCS_ILLUMINANTS["CIE 1931 2 Degree Standard Observer"][
                    "ICC D50"
                ],
            )
        ),
    )
)

CCS_ILLUMINANT_COLORCHECKER24_BEFORE_NOV2014: NDArray = CCS_ILLUMINANTS[
    "CIE 1931 2 Degree Standard Observer"
]["ICC D50"]
"""*ColorChecker24 - Before November 2014* illuminant."""

CCS_COLORCHECKER24_BEFORE_NOV2014: ColourChecker = ColourChecker(
    "ColorChecker24 - Before November 2014",
    DATA_COLORCHECKER24_BEFORE_NOV2014,
    CCS_ILLUMINANT_COLORCHECKER24_BEFORE_NOV2014,
)
"""
Reference *ColorChecker Classic* data from *X-Rite (2015)*.

Notes
-----
-   The rounded *ColorChecker24 - Before November 2014* values should match the
    *ColorChecker Classic 2005* values. They are given for reference of the
    original *CIE L\\*a\\*b\\** colourspace values.
"""

DATA_COLORCHECKER24_AFTER_NOV2014_CIE_LAB: Dict = dict(
    [
        ("dark skin", np.array([37.54, 14.37, 14.92])),
        ("light skin", np.array([64.66, 19.27, 17.5])),
        ("blue sky", np.array([49.32, -3.82, -22.54])),
        ("foliage", np.array([43.46, -12.74, 22.72])),
        ("blue flower", np.array([54.94, 9.61, -24.79])),
        ("bluish green", np.array([70.48, -32.26, -0.37])),
        ("orange", np.array([62.73, 35.83, 56.5])),
        ("purplish blue", np.array([39.43, 10.75, -45.17])),
        ("moderate red", np.array([50.57, 48.64, 16.67])),
        ("purple", np.array([30.1, 22.54, -20.87])),
        ("yellow green", np.array([71.77, -24.13, 58.19])),
        ("orange yellow", np.array([71.51, 18.24, 67.37])),
        ("blue", np.array([28.37, 15.42, -49.8])),
        ("green", np.array([54.38, -39.72, 32.27])),
        ("red", np.array([42.43, 51.05, 28.62])),
        ("yellow", np.array([81.8, 2.67, 80.41])),
        ("magenta", np.array([50.63, 51.28, -14.12])),
        ("cyan", np.array([49.57, -29.71, -28.32])),
        ("white 9.5 (.05 D)", np.array([95.19, -1.03, 2.93])),
        ("neutral 8 (.23 D)", np.array([81.29, -0.57, 0.44])),
        ("neutral 6.5 (.44 D)", np.array([66.89, -0.75, -0.06])),
        ("neutral 5 (.70 D)", np.array([50.76, -0.13, 0.14])),
        ("neutral 3.5 (1.05 D)", np.array([35.63, -0.46, -0.48])),
        ("black 2 (1.5 D)", np.array([20.64, 0.07, -0.46])),
    ]
)

DATA_COLORCHECKER24_AFTER_NOV2014: Dict = dict(
    zip(
        SAMPLE_LABELS_COLORCHECKER_CLASSIC,
        XYZ_to_xyY(
            Lab_to_XYZ(
                list(DATA_COLORCHECKER24_AFTER_NOV2014_CIE_LAB.values()),
                CCS_ILLUMINANTS["CIE 1931 2 Degree Standard Observer"][
                    "ICC D50"
                ],
            )
        ),
    )
)

CCS_ILLUMINANT_COLORCHECKER24_AFTER_NOV2014: NDArray = CCS_ILLUMINANTS[
    "CIE 1931 2 Degree Standard Observer"
]["ICC D50"]
"""*ColorChecker24 - After November 2014* illuminant."""

CCS_COLORCHECKER24_AFTER_NOV2014: ColourChecker = ColourChecker(
    "ColorChecker24 - After November 2014",
    DATA_COLORCHECKER24_AFTER_NOV2014,
    CCS_ILLUMINANT_COLORCHECKER24_AFTER_NOV2014,
)
"""
Reference *ColorChecker Classic* data from *X-Rite (2015)* and matching the
*ColorChecker Classic* edition after November 2014.
"""

CCS_COLOURCHECKERS: CaseInsensitiveMapping = CaseInsensitiveMapping(
    {
        "ColorChecker 1976": CCS_COLORCHECKER1976,
        "ColorChecker 2005": CCS_COLORCHECKER2005,
        "BabelColor Average": CCS_BABELCOLOR_AVERAGE,
        "ColorChecker24 - Before November 2014": CCS_COLORCHECKER24_BEFORE_NOV2014,
        "ColorChecker24 - After November 2014": CCS_COLORCHECKER24_AFTER_NOV2014,
    }
)
CCS_COLOURCHECKERS.__doc__ = """
Chromaticity coordinates of the colour checkers.

References
----------
:cite:`BabelColor2012b`, :cite:`BabelColor2012c`, :cite:`X-Rite2016`

Aliases:

-   'babel_average': 'BabelColor Average'
-   'cc2005': 'ColorChecker 2005'
-   'ccb2014': 'ColorChecker24 - Before November 2014'
-   'cca2014': 'ColorChecker24 - After November 2014'
"""
CCS_COLOURCHECKERS["babel_average"] = CCS_COLOURCHECKERS["BabelColor Average"]
CCS_COLOURCHECKERS["cc2005"] = CCS_COLOURCHECKERS["ColorChecker 2005"]
CCS_COLOURCHECKERS["ccb2014"] = CCS_COLOURCHECKERS[
    "ColorChecker24 - Before November 2014"
]
CCS_COLOURCHECKERS["cca2014"] = CCS_COLOURCHECKERS[
    "ColorChecker24 - After November 2014"
]



"""Defines the unit tests for the :mod:`colour.models.rgb.derivation` module."""

import numpy as np
import re
import unittest
from itertools import permutations

from colour.models import (
    normalised_primary_matrix,
    chromatically_adapted_primaries,
    primaries_whitepoint,
    RGB_luminance_equation,
    RGB_luminance,
)
from colour.models.rgb.derivation import xy_to_z
from colour.utilities import ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "Testxy_to_z",
    "TestNormalisedPrimaryMatrix",
    "TestChromaticallyAdaptedPrimaries",
    "TestPrimariesWhitepoint",
    "TestRGBLuminanceEquation",
    "TestRGBLuminance",
]


class Testxy_to_z(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.derivation.xy_to_z` definition unit
    tests methods.
    """

    def test_xy_to_z(self):
        """Test :func:`colour.models.rgb.derivation.xy_to_z` definition."""

        np.testing.assert_almost_equal(
            xy_to_z(np.array([0.2500, 0.2500])), 0.50000000, decimal=7
        )

        np.testing.assert_almost_equal(
            xy_to_z(np.array([0.0001, -0.0770])), 1.07690000, decimal=7
        )

        np.testing.assert_almost_equal(
            xy_to_z(np.array([0.0000, 1.0000])), 0.00000000, decimal=7
        )

    def test_n_dimensional_xy_to_z(self):
        """
        Test :func:`colour.models.rgb.derivation.xy_to_z` definition
        n-dimensional arrays support.
        """

        xy = np.array([0.25, 0.25])
        z = xy_to_z(xy)

        xy = np.tile(xy, (6, 1))
        z = np.tile(
            z,
            6,
        )
        np.testing.assert_almost_equal(xy_to_z(xy), z, decimal=7)

        xy = np.reshape(xy, (2, 3, 2))
        z = np.reshape(z, (2, 3))
        np.testing.assert_almost_equal(xy_to_z(xy), z, decimal=7)

    @ignore_numpy_errors
    def test_nan_xy_to_z(self):
        """
        Test :func:`colour.models.rgb.derivation.xy_to_z` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=2))
        for case in cases:
            xy_to_z(case)


class TestNormalisedPrimaryMatrix(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.derivation.normalised_primary_matrix`
    definition unit tests methods.
    """

    def test_normalised_primary_matrix(self):
        """
        Test :func:`colour.models.rgb.derivation.normalised_primary_matrix`
        definition.
        """

        np.testing.assert_almost_equal(
            normalised_primary_matrix(
                np.array(
                    [0.73470, 0.26530, 0.00000, 1.00000, 0.00010, -0.07700]
                ),
                np.array([0.32168, 0.33767]),
            ),
            np.array(
                [
                    [0.95255240, 0.00000000, 0.00009368],
                    [0.34396645, 0.72816610, -0.07213255],
                    [0.00000000, 0.00000000, 1.00882518],
                ]
            ),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            normalised_primary_matrix(
                np.array([0.640, 0.330, 0.300, 0.600, 0.150, 0.060]),
                np.array([0.3127, 0.3290]),
            ),
            np.array(
                [
                    [0.41239080, 0.35758434, 0.18048079],
                    [0.21263901, 0.71516868, 0.07219232],
                    [0.01933082, 0.11919478, 0.95053215],
                ]
            ),
            decimal=7,
        )

    @ignore_numpy_errors
    def test_nan_normalised_primary_matrix(self):
        """
        Test :func:`colour.models.rgb.derivation.normalised_primary_matrix`
        definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=2))
        for case in cases:
            P = np.array(np.vstack([case, case, case]))
            W = np.array(case)
            try:
                normalised_primary_matrix(P, W)
            except np.linalg.linalg.LinAlgError:
                pass


class TestChromaticallyAdaptedPrimaries(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.derivation.\
chromatically_adapted_primaries` definition unit tests methods.
    """

    def test_chromatically_adapted_primaries(self):
        """
        Test :func:`colour.models.rgb.derivation.\
chromatically_adapted_primaries` definition.
        """

        np.testing.assert_almost_equal(
            chromatically_adapted_primaries(
                np.array(
                    [0.73470, 0.26530, 0.00000, 1.00000, 0.00010, -0.07700]
                ),
                np.array([0.32168, 0.33767]),
                np.array([0.34570, 0.35850]),
            ),
            np.array(
                [
                    [0.73431182, 0.26694964],
                    [0.02211963, 0.98038009],
                    [-0.05880375, -0.12573056],
                ]
            ),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            chromatically_adapted_primaries(
                np.array([0.640, 0.330, 0.300, 0.600, 0.150, 0.060]),
                np.array([0.31270, 0.32900]),
                np.array([0.34570, 0.35850]),
            ),
            np.array(
                [
                    [0.64922534, 0.33062196],
                    [0.32425276, 0.60237128],
                    [0.15236177, 0.06118676],
                ]
            ),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            chromatically_adapted_primaries(
                np.array([0.640, 0.330, 0.300, 0.600, 0.150, 0.060]),
                np.array([0.31270, 0.32900]),
                np.array([0.34570, 0.35850]),
                "Bradford",
            ),
            np.array(
                [
                    [0.64844144, 0.33085331],
                    [0.32119518, 0.59784434],
                    [0.15589322, 0.06604921],
                ]
            ),
            decimal=7,
        )

    @ignore_numpy_errors
    def test_nan_chromatically_adapted_primaries(self):
        """
        Test :func:`colour.models.rgb.derivation.\
chromatically_adapted_primaries` definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=2))
        for case in cases:
            P = np.array(np.vstack([case, case, case]))
            W = np.array(case)
            chromatically_adapted_primaries(P, W, W)


class TestPrimariesWhitepoint(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.derivation.primaries_whitepoint`
    definition unit tests methods.
    """

    def test_primaries_whitepoint(self):
        """
        Test :func:`colour.models.rgb.derivation.primaries_whitepoint`
        definition.
        """

        P, W = primaries_whitepoint(
            np.array(
                [
                    [0.95255240, 0.00000000, 0.00009368],
                    [0.34396645, 0.72816610, -0.07213255],
                    [0.00000000, 0.00000000, 1.00882518],
                ]
            )
        )
        np.testing.assert_almost_equal(
            P,
            np.array(
                [
                    [0.73470, 0.26530],
                    [0.00000, 1.00000],
                    [0.00010, -0.07700],
                ]
            ),
            decimal=7,
        )
        np.testing.assert_almost_equal(
            W, np.array([0.32168, 0.33767]), decimal=7
        )

        P, W = primaries_whitepoint(
            np.array(
                [
                    [0.41240000, 0.35760000, 0.18050000],
                    [0.21260000, 0.71520000, 0.07220000],
                    [0.01930000, 0.11920000, 0.95050000],
                ]
            )
        )
        np.testing.assert_almost_equal(
            P,
            np.array(
                [
                    [0.64007450, 0.32997051],
                    [0.30000000, 0.60000000],
                    [0.15001662, 0.06000665],
                ]
            ),
            decimal=7,
        )
        np.testing.assert_almost_equal(
            W, np.array([0.31271591, 0.32900148]), decimal=7
        )

    @ignore_numpy_errors
    def test_nan_primaries_whitepoint(self):
        """
        Test :func:`colour.models.rgb.derivation.primaries_whitepoint`
        definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            M = np.array(np.vstack([case, case, case]))
            primaries_whitepoint(M)


class TestRGBLuminanceEquation(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.derivation.RGB_luminance_equation`
    definition unit tests methods.
    """

    def test_RGB_luminance_equation(self):
        """
        Test :func:`colour.models.rgb.derivation.RGB_luminance_equation`
        definition.
        """

        self.assertIsInstance(
            RGB_luminance_equation(
                np.array(
                    [0.73470, 0.26530, 0.00000, 1.00000, 0.00010, -0.07700]
                ),
                np.array([0.32168, 0.33767]),
            ),
            str,
        )

        # TODO: Simplify that monster.
        pattern = (
            "Y\\s?=\\s?[-+]?[0-9]*\\.?[0-9]+([eE][-+]?[0-9]+)?."
            "\\(R\\)\\s?[+-]\\s?[-+]?[0-9]*\\.?[0-9]+([eE][-+]?[0-9]+)?."
            "\\(G\\)\\s?[+-]\\s?[-+]?[0-9]*\\.?[0-9]+([eE][-+]?[0-9]+)?."
            "\\(B\\)"
        )
        P = np.array([0.73470, 0.26530, 0.00000, 1.00000, 0.00010, -0.07700])
        self.assertTrue(
            re.match(
                pattern,
                RGB_luminance_equation(P, np.array([0.32168, 0.33767])),
            )
        )


class TestRGBLuminance(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.derivation.RGB_luminance` definition
    unit tests methods.
    """

    def test_RGB_luminance(self):
        """
        Test :func:`colour.models.rgb.derivation.RGB_luminance`
        definition.
        """

        self.assertAlmostEqual(
            RGB_luminance(
                np.array([0.18, 0.18, 0.18]),
                np.array(
                    [0.73470, 0.26530, 0.00000, 1.00000, 0.00010, -0.07700]
                ),
                np.array([0.32168, 0.33767]),
            ),
            0.18000000,
            places=7,
        )

        self.assertAlmostEqual(
            RGB_luminance(
                np.array([0.21959402, 0.06986677, 0.04703877]),
                np.array(
                    [0.73470, 0.26530, 0.00000, 1.00000, 0.00010, -0.07700]
                ),
                np.array([0.32168, 0.33767]),
            ),
            0.123014562384318,
            places=7,
        )

        self.assertAlmostEqual(
            RGB_luminance(
                np.array([0.45620519, 0.03081071, 0.04091952]),
                np.array([0.6400, 0.3300, 0.3000, 0.6000, 0.1500, 0.0600]),
                np.array([0.31270, 0.32900]),
            ),
            0.121995947729870,
            places=7,
        )

    def test_n_dimensional_RGB_luminance(self):
        """
        Test :func:`colour.models.rgb.derivation.RGB_luminance` definition
        n_dimensional arrays support.
        """

        RGB = (np.array([0.18, 0.18, 0.18]),)
        P = (
            np.array([0.73470, 0.26530, 0.00000, 1.00000, 0.00010, -0.07700]),
        )
        W = np.array([0.32168, 0.33767])
        Y = RGB_luminance(RGB, P, W)

        RGB = np.tile(RGB, (6, 1))
        Y = np.tile(Y, 6)
        np.testing.assert_almost_equal(RGB_luminance(RGB, P, W), Y)

        RGB = np.reshape(RGB, (2, 3, 3))
        Y = np.reshape(Y, (2, 3))
        np.testing.assert_almost_equal(RGB_luminance(RGB, P, W), Y)

    @ignore_numpy_errors
    def test_nan_RGB_luminance(self):
        """
        Test :func:`colour.models.rgb.derivation.RGB_luminance`
        definition nan support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            RGB = np.array(case)
            P = np.array(np.vstack([case[0:2], case[0:2], case[0:2]]))
            W = np.array(case[0:2])
            try:
                RGB_luminance(RGB, P, W)
            except np.linalg.linalg.LinAlgError:
                pass


if __name__ == "__main__":
    unittest.main()



"""Showcases *LLAB(l:c)* colour appearance model computations."""

import numpy as np

import colour
from colour.appearance.llab import CAM_ReferenceSpecification_LLAB
from colour.utilities import message_box

message_box('"LLAB(l:c)" Colour Appearance Model Computations')

XYZ = np.array([19.01, 20.00, 21.78])
XYZ_0 = np.array([95.05, 100.00, 108.88])
Y_b = 20.0
L = 318.31
surround = colour.VIEWING_CONDITIONS_LLAB["ref_average_4_minus"]
message_box(
    f'Converting to the  "LLAB(l:c)" colour appearance model specification '
    f"using given parameters:\n\n"
    f"\tXYZ: {XYZ}\n"
    f"\tXYZ_0: {XYZ_0}\n"
    f"\tY_b: {Y_b}\n"
    f"\tL: {L}\n"
    f"\tsurround: {surround}"
)
specification = colour.XYZ_to_LLAB(XYZ, XYZ_0, Y_b, L, surround)
print(specification)

print("\n")

message_box(
    'Broadcasting the current output "LLAB(l:c)" colour appearance '
    "model specification to the reference specification.\n"
    "The intent of this reference specification is to provide names "
    'as closest as possible to the "Mark D. Fairchild" reference.\n'
    "The current output specification is meant to be consistent with "
    "the other colour appearance model specification by using same "
    "argument names for consistency wherever possible."
)

print(CAM_ReferenceSpecification_LLAB(*specification.values))



"""
Defines the unit tests for the
:mod:`colour.models.rgb.transfer_functions.itur_bt_709` module.
"""

import numpy as np
import unittest

from colour.models.rgb.transfer_functions import oetf_BT709, oetf_inverse_BT709
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestOetf_BT709",
    "TestOetf_inverse_BT709",
]


class TestOetf_BT709(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_709.oetf_BT709`
    definition unit tests methods.
    """

    def test_oetf_BT709(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_709.\
oetf_BT709` definition.
        """

        self.assertAlmostEqual(oetf_BT709(0.0), 0.0, places=7)

        self.assertAlmostEqual(oetf_BT709(0.015), 0.067500000000000, places=7)

        self.assertAlmostEqual(oetf_BT709(0.18), 0.409007728864150, places=7)

        self.assertAlmostEqual(oetf_BT709(1.0), 1.0, places=7)

    def test_n_dimensional_oetf_BT709(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_709.\
oetf_BT709` definition n-dimensional arrays support.
        """

        L = 0.18
        V = oetf_BT709(L)

        L = np.tile(L, 6)
        V = np.tile(V, 6)
        np.testing.assert_almost_equal(oetf_BT709(L), V, decimal=7)

        L = np.reshape(L, (2, 3))
        V = np.reshape(V, (2, 3))
        np.testing.assert_almost_equal(oetf_BT709(L), V, decimal=7)

        L = np.reshape(L, (2, 3, 1))
        V = np.reshape(V, (2, 3, 1))
        np.testing.assert_almost_equal(oetf_BT709(L), V, decimal=7)

    def test_domain_range_scale_oetf_BT709(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_709.\
oetf_BT709` definition domain and range scale support.
        """

        L = 0.18
        V = oetf_BT709(L)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    oetf_BT709(L * factor), V * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_oetf_BT709(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_709.\
oetf_BT709` definition nan support.
        """

        oetf_BT709(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestOetf_inverse_BT709(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.itur_bt_709.\
oetf_inverse_BT709` definition unit tests methods.
    """

    def test_oetf_inverse_BT709(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_709.\
oetf_inverse_BT709` definition.
        """

        self.assertAlmostEqual(oetf_inverse_BT709(0.0), 0.0, places=7)

        self.assertAlmostEqual(
            oetf_inverse_BT709(0.067500000000000), 0.015, places=7
        )

        self.assertAlmostEqual(
            oetf_inverse_BT709(0.409007728864150), 0.18, places=7
        )

        self.assertAlmostEqual(oetf_inverse_BT709(1.0), 1.0, places=7)

    def test_n_dimensional_oetf_inverse_BT709(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_709.\
oetf_inverse_BT709` definition n-dimensional arrays support.
        """

        V = 0.409007728864150
        L = oetf_inverse_BT709(V)

        V = np.tile(V, 6)
        L = np.tile(L, 6)
        np.testing.assert_almost_equal(oetf_inverse_BT709(V), L, decimal=7)

        V = np.reshape(V, (2, 3))
        L = np.reshape(L, (2, 3))
        np.testing.assert_almost_equal(oetf_inverse_BT709(V), L, decimal=7)

        V = np.reshape(V, (2, 3, 1))
        L = np.reshape(L, (2, 3, 1))
        np.testing.assert_almost_equal(oetf_inverse_BT709(V), L, decimal=7)

    def test_domain_range_scale_oetf_inverse_BT709(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_709.\
oetf_inverse_BT709` definition domain and range scale support.
        """

        V = 0.409007728864150
        L = oetf_inverse_BT709(V)

        d_r = (("reference", 1), ("1", 1), ("100", 100))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    oetf_inverse_BT709(V * factor), L * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_oetf_inverse_BT709(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.itur_bt_709.\
oetf_inverse_BT709` definition nan support.
        """

        oetf_inverse_BT709(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


if __name__ == "__main__":
    unittest.main()



"""Defines the unit tests for the :mod:`colour.models.cie_lab` module."""

import numpy as np
import unittest
from itertools import permutations

from colour.models import XYZ_to_Lab, Lab_to_XYZ, Lab_to_LCHab, LCHab_to_Lab
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestXYZ_to_Lab",
    "TestLab_to_XYZ",
    "TestLab_to_LCHab",
    "TestLCHab_to_Lab",
]


class TestXYZ_to_Lab(unittest.TestCase):
    """
    Define :func:`colour.models.cie_lab.XYZ_to_Lab` definition unit tests
    methods.
    """

    def test_XYZ_to_Lab(self):
        """Test :func:`colour.models.cie_lab.XYZ_to_Lab` definition."""

        np.testing.assert_almost_equal(
            XYZ_to_Lab(np.array([0.20654008, 0.12197225, 0.05136952])),
            np.array([41.52787529, 52.63858304, 26.92317922]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_Lab(np.array([0.14222010, 0.23042768, 0.10495772])),
            np.array([55.11636304, -41.08791787, 30.91825778]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_Lab(np.array([0.07818780, 0.06157201, 0.28099326])),
            np.array([29.80565520, 20.01830466, -48.34913874]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_Lab(
                np.array([0.20654008, 0.12197225, 0.05136952]),
                np.array([0.44757, 0.40745]),
            ),
            np.array([41.52787529, 38.48089305, -5.73295122]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_Lab(
                np.array([0.20654008, 0.12197225, 0.05136952]),
                np.array([0.34570, 0.35850]),
            ),
            np.array([41.52787529, 51.19354174, 19.91843098]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            XYZ_to_Lab(
                np.array([0.20654008, 0.12197225, 0.05136952]),
                np.array([0.34570, 0.35850, 1.00000]),
            ),
            np.array([41.52787529, 51.19354174, 19.91843098]),
            decimal=7,
        )

    def test_n_dimensional_XYZ_to_Lab(self):
        """
        Test :func:`colour.models.cie_lab.XYZ_to_Lab` definition n-dimensional
        support.
        """

        XYZ = np.array([0.20654008, 0.12197225, 0.05136952])
        illuminant = np.array([0.31270, 0.32900])
        Lab = XYZ_to_Lab(XYZ, illuminant)

        XYZ = np.tile(XYZ, (6, 1))
        Lab = np.tile(Lab, (6, 1))
        np.testing.assert_almost_equal(
            XYZ_to_Lab(XYZ, illuminant), Lab, decimal=7
        )

        illuminant = np.tile(illuminant, (6, 1))
        np.testing.assert_almost_equal(
            XYZ_to_Lab(XYZ, illuminant), Lab, decimal=7
        )

        XYZ = np.reshape(XYZ, (2, 3, 3))
        illuminant = np.reshape(illuminant, (2, 3, 2))
        Lab = np.reshape(Lab, (2, 3, 3))
        np.testing.assert_almost_equal(
            XYZ_to_Lab(XYZ, illuminant), Lab, decimal=7
        )

    def test_domain_range_scale_XYZ_to_Lab(self):
        """
        Test :func:`colour.models.cie_lab.XYZ_to_Lab` definition
        domain and range scale support.
        """

        XYZ = np.array([0.20654008, 0.12197225, 0.05136952])
        illuminant = np.array([0.31270, 0.32900])
        Lab = XYZ_to_Lab(XYZ, illuminant)

        d_r = (("reference", 1, 1), ("1", 1, 0.01), ("100", 100, 1))
        for scale, factor_a, factor_b in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    XYZ_to_Lab(XYZ * factor_a, illuminant),
                    Lab * factor_b,
                    decimal=7,
                )

    @ignore_numpy_errors
    def test_nan_XYZ_to_Lab(self):
        """Test :func:`colour.models.cie_lab.XYZ_to_Lab` definition nan support."""

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            XYZ = np.array(case)
            illuminant = np.array(case[0:2])
            XYZ_to_Lab(XYZ, illuminant)


class TestLab_to_XYZ(unittest.TestCase):
    """
    Define :func:`colour.models.cie_lab.Lab_to_XYZ` definition unit tests
    methods.
    """

    def test_Lab_to_XYZ(self):
        """Test :func:`colour.models.cie_lab.Lab_to_XYZ` definition."""

        np.testing.assert_almost_equal(
            Lab_to_XYZ(np.array([41.52787529, 52.63858304, 26.92317922])),
            np.array([0.20654008, 0.12197225, 0.05136952]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            Lab_to_XYZ(np.array([55.11636304, -41.08791787, 30.91825778])),
            np.array([0.14222010, 0.23042768, 0.10495772]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            Lab_to_XYZ(np.array([29.80565520, 20.01830466, -48.34913874])),
            np.array([0.07818780, 0.06157201, 0.28099326]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            Lab_to_XYZ(
                np.array([41.52787529, 38.48089305, -5.73295122]),
                np.array([0.44757, 0.40745]),
            ),
            np.array([0.20654008, 0.12197225, 0.05136952]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            Lab_to_XYZ(
                np.array([41.52787529, 51.19354174, 19.91843098]),
                np.array([0.34570, 0.35850]),
            ),
            np.array([0.20654008, 0.12197225, 0.05136952]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            Lab_to_XYZ(
                np.array([41.52787529, 51.19354174, 19.91843098]),
                np.array([0.34570, 0.35850, 1.00000]),
            ),
            np.array([0.20654008, 0.12197225, 0.05136952]),
            decimal=7,
        )

    def test_n_dimensional_Lab_to_XYZ(self):
        """
        Test :func:`colour.models.cie_lab.Lab_to_XYZ` definition n-dimensional
        support.
        """

        Lab = np.array([41.52787529, 52.63858304, 26.92317922])
        illuminant = np.array([0.31270, 0.32900])
        XYZ = Lab_to_XYZ(Lab, illuminant)

        Lab = np.tile(Lab, (6, 1))
        XYZ = np.tile(XYZ, (6, 1))
        np.testing.assert_almost_equal(
            Lab_to_XYZ(Lab, illuminant), XYZ, decimal=7
        )

        illuminant = np.tile(illuminant, (6, 1))
        np.testing.assert_almost_equal(
            Lab_to_XYZ(Lab, illuminant), XYZ, decimal=7
        )

        Lab = np.reshape(Lab, (2, 3, 3))
        illuminant = np.reshape(illuminant, (2, 3, 2))
        XYZ = np.reshape(XYZ, (2, 3, 3))
        np.testing.assert_almost_equal(
            Lab_to_XYZ(Lab, illuminant), XYZ, decimal=7
        )

    def test_domain_range_scale_Lab_to_XYZ(self):
        """
        Test :func:`colour.models.cie_lab.Lab_to_XYZ` definition
        domain and range scale support.
        """

        Lab = np.array([41.52787529, 52.63858304, 26.92317922])
        illuminant = np.array([0.31270, 0.32900])
        XYZ = Lab_to_XYZ(Lab, illuminant)

        d_r = (("reference", 1, 1), ("1", 0.01, 1), ("100", 1, 100))
        for scale, factor_a, factor_b in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    Lab_to_XYZ(Lab * factor_a, illuminant),
                    XYZ * factor_b,
                    decimal=7,
                )

    @ignore_numpy_errors
    def test_nan_Lab_to_XYZ(self):
        """Test :func:`colour.models.cie_lab.Lab_to_XYZ` definition nan support."""

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            Lab = np.array(case)
            illuminant = np.array(case[0:2])
            Lab_to_XYZ(Lab, illuminant)


class TestLab_to_LCHab(unittest.TestCase):
    """
    Define :func:`colour.models.cie_lab.Lab_to_LCHab` definition unit tests
    methods.
    """

    def test_Lab_to_LCHab(self):
        """Test :func:`colour.models.cie_lab.Lab_to_LCHab` definition."""

        np.testing.assert_almost_equal(
            Lab_to_LCHab(np.array([41.52787529, 52.63858304, 26.92317922])),
            np.array([41.52787529, 59.12425901, 27.08848784]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            Lab_to_LCHab(np.array([55.11636304, -41.08791787, 30.91825778])),
            np.array([55.11636304, 51.42135412, 143.03889556]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            Lab_to_LCHab(np.array([29.80565520, 20.01830466, -48.34913874])),
            np.array([29.80565520, 52.32945383, 292.49133666]),
            decimal=7,
        )

    def test_n_dimensional_Lab_to_LCHab(self):
        """
        Test :func:`colour.models.cie_lab.Lab_to_LCHab` definition
        n-dimensional arrays support.
        """

        Lab = np.array([41.52787529, 52.63858304, 26.92317922])
        LCHab = Lab_to_LCHab(Lab)

        Lab = np.tile(Lab, (6, 1))
        LCHab = np.tile(LCHab, (6, 1))
        np.testing.assert_almost_equal(Lab_to_LCHab(Lab), LCHab, decimal=7)

        Lab = np.reshape(Lab, (2, 3, 3))
        LCHab = np.reshape(LCHab, (2, 3, 3))
        np.testing.assert_almost_equal(Lab_to_LCHab(Lab), LCHab, decimal=7)

    def test_domain_range_scale_Lab_to_LCHab(self):
        """
        Test :func:`colour.models.cie_lab.Lab_to_LCHab` definition domain and
        range scale support.
        """

        Lab = np.array([41.52787529, 52.63858304, 26.92317922])
        LCHab = Lab_to_LCHab(Lab)

        d_r = (
            ("reference", 1, 1),
            ("1", 0.01, np.array([0.01, 0.01, 1 / 360])),
            ("100", 1, np.array([1, 1, 1 / 3.6])),
        )
        for scale, factor_a, factor_b in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    Lab_to_LCHab(Lab * factor_a), LCHab * factor_b, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_Lab_to_LCHab(self):
        """
        Test :func:`colour.models.cie_lab.Lab_to_LCHab` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            Lab = np.array(case)
            Lab_to_LCHab(Lab)


class TestLCHab_to_Lab(unittest.TestCase):
    """
    Define :func:`colour.models.cie_lab.LCHab_to_Lab` definition unit tests
    methods.
    """

    def test_LCHab_to_Lab(self):
        """Test :func:`colour.models.cie_lab.LCHab_to_Lab` definition."""

        np.testing.assert_almost_equal(
            LCHab_to_Lab(np.array([41.52787529, 59.12425901, 27.08848784])),
            np.array([41.52787529, 52.63858304, 26.92317922]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            LCHab_to_Lab(np.array([55.11636304, 51.42135412, 143.03889556])),
            np.array([55.11636304, -41.08791787, 30.91825778]),
            decimal=7,
        )

        np.testing.assert_almost_equal(
            LCHab_to_Lab(np.array([29.80565520, 52.32945383, 292.49133666])),
            np.array([29.80565520, 20.01830466, -48.34913874]),
            decimal=7,
        )

    def test_n_dimensional_LCHab_to_Lab(self):
        """
        Test :func:`colour.models.cie_lab.LCHab_to_Lab` definition
        n-dimensional arrays support.
        """

        LCHab = np.array([41.52787529, 59.12425901, 27.08848784])
        Lab = LCHab_to_Lab(LCHab)

        LCHab = np.tile(LCHab, (6, 1))
        Lab = np.tile(Lab, (6, 1))
        np.testing.assert_almost_equal(LCHab_to_Lab(LCHab), Lab, decimal=7)

        LCHab = np.reshape(LCHab, (2, 3, 3))
        Lab = np.reshape(Lab, (2, 3, 3))
        np.testing.assert_almost_equal(LCHab_to_Lab(LCHab), Lab, decimal=7)

    def test_domain_range_scale_LCHab_to_Lab(self):
        """
        Test :func:`colour.models.cie_lab.LCHab_to_Lab` definition domain and
        range scale support.
        """

        LCHab = np.array([41.52787529, 59.12425901, 27.08848784])
        Lab = LCHab_to_Lab(LCHab)

        d_r = (
            ("reference", 1, 1),
            ("1", np.array([0.01, 0.01, 1 / 360]), 0.01),
            ("100", np.array([1, 1, 1 / 3.6]), 1),
        )
        for scale, factor_a, factor_b in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    LCHab_to_Lab(LCHab * factor_a), Lab * factor_b, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_LCHab_to_Lab(self):
        """
        Test :func:`colour.models.cie_lab.LCHab_to_Lab` definition nan
        support.
        """

        cases = [-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]
        cases = set(permutations(cases * 3, r=3))
        for case in cases:
            LCHab = np.array(case)
            LCHab_to_Lab(LCHab)


if __name__ == "__main__":
    unittest.main()



"""
Defines the unit tests for the :mod:`colour.models.rgb.transfer_functions.dcdm`
module.
"""

import numpy as np
import unittest

from colour.models.rgb.transfer_functions import eotf_inverse_DCDM, eotf_DCDM
from colour.utilities import domain_range_scale, ignore_numpy_errors

__author__ = "Colour Developers"
__copyright__ = "Copyright 2013 Colour Developers"
__license__ = "New BSD License - https://opensource.org/licenses/BSD-3-Clause"
__maintainer__ = "Colour Developers"
__email__ = "colour-developers@colour-science.org"
__status__ = "Production"

__all__ = [
    "TestEotf_inverse_DCDM",
    "TestEotf_DCDM",
]


class TestEotf_inverse_DCDM(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.dcdm.eotf_inverse_DCDM`
    definition unit tests methods.
    """

    def test_eotf_inverse_DCDM(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.\
dcdm.eotf_inverse_DCDM` definition.
        """

        self.assertAlmostEqual(eotf_inverse_DCDM(0.0), 0.0, places=7)

        self.assertAlmostEqual(eotf_inverse_DCDM(0.18), 0.11281861, places=7)

        self.assertAlmostEqual(eotf_inverse_DCDM(1.0), 0.21817973, places=7)

        self.assertEqual(eotf_inverse_DCDM(0.18, out_int=True), 462)

    def test_n_dimensional_eotf_inverse_DCDM(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.dcdm.\
eotf_inverse_DCDM` definition n-dimensional arrays support.
        """

        XYZ = 0.18
        XYZ_p = eotf_inverse_DCDM(XYZ)

        XYZ = np.tile(XYZ, 6)
        XYZ_p = np.tile(XYZ_p, 6)
        np.testing.assert_almost_equal(
            eotf_inverse_DCDM(XYZ), XYZ_p, decimal=7
        )

        XYZ = np.reshape(XYZ, (2, 3))
        XYZ_p = np.reshape(XYZ_p, (2, 3))
        np.testing.assert_almost_equal(
            eotf_inverse_DCDM(XYZ), XYZ_p, decimal=7
        )

        XYZ = np.reshape(XYZ, (2, 3, 1))
        XYZ_p = np.reshape(XYZ_p, (2, 3, 1))
        np.testing.assert_almost_equal(
            eotf_inverse_DCDM(XYZ), XYZ_p, decimal=7
        )

    def test_domain_range_scale_eotf_inverse_DCDM(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.\
dcdm.eotf_inverse_DCDM` definition domain and range scale support.
        """

        XYZ = 0.18
        XYZ_p = eotf_inverse_DCDM(XYZ)

        d_r = (("reference", 1), ("1", 1), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    eotf_inverse_DCDM(XYZ * factor), XYZ_p * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_eotf_inverse_DCDM(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.dcdm.\
eotf_inverse_DCDM` definition nan support.
        """

        eotf_inverse_DCDM(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestEotf_DCDM(unittest.TestCase):
    """
    Define :func:`colour.models.rgb.transfer_functions.dcdm.eotf_DCDM`
    definition unit tests methods.
    """

    def test_eotf_DCDM(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.dcdm.eotf_DCDM`
        definition.
        """

        self.assertAlmostEqual(eotf_DCDM(0.0), 0.0, places=7)

        self.assertAlmostEqual(eotf_DCDM(0.11281861), 0.18, places=7)

        self.assertAlmostEqual(eotf_DCDM(0.21817973), 1.0, places=7)

        np.testing.assert_allclose(
            eotf_DCDM(462, in_int=True), 0.18, atol=0.00001, rtol=0.00001
        )

    def test_n_dimensional_eotf_DCDM(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.dcdm.eotf_DCDM`
        definition n-dimensional arrays support.
        """

        XYZ_p = 0.11281861
        XYZ = eotf_DCDM(XYZ_p)

        XYZ_p = np.tile(XYZ_p, 6)
        XYZ = np.tile(XYZ, 6)
        np.testing.assert_almost_equal(eotf_DCDM(XYZ_p), XYZ, decimal=7)

        XYZ_p = np.reshape(XYZ_p, (2, 3))
        XYZ = np.reshape(XYZ, (2, 3))
        np.testing.assert_almost_equal(eotf_DCDM(XYZ_p), XYZ, decimal=7)

        XYZ_p = np.reshape(XYZ_p, (2, 3, 1))
        XYZ = np.reshape(XYZ, (2, 3, 1))
        np.testing.assert_almost_equal(eotf_DCDM(XYZ_p), XYZ, decimal=7)

    def test_domain_range_scale_eotf_DCDM(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.dcdm.eotf_DCDM`
        definition domain and range scale support.
        """

        XYZ_p = 0.11281861
        XYZ = eotf_DCDM(XYZ_p)

        d_r = (("reference", 1), ("1", 1), ("100", 1))
        for scale, factor in d_r:
            with domain_range_scale(scale):
                np.testing.assert_almost_equal(
                    eotf_DCDM(XYZ_p * factor), XYZ * factor, decimal=7
                )

    @ignore_numpy_errors
    def test_nan_eotf_DCDM(self):
        """
        Test :func:`colour.models.rgb.transfer_functions.dcdm.eotf_DCDM`
        definition nan support.
        """

        eotf_DCDM(np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


if __name__ == "__main__":
    unittest.main()
